SystemOrganization addCategory: #'SqSquare-Server-Snapshot'!
SystemOrganization addCategory: #'SqSquare-Server-Service'!
SystemOrganization addCategory: #'SqSquare-Server'!
SystemOrganization addCategory: #'SqSquare-Server-Storage'!
SystemOrganization addCategory: #'SqSquare-Server-Storage-Tests'!
SystemOrganization addCategory: #'SqSquare-Server-AudioChat'!
SystemOrganization addCategory: #'SqSquare-Server-Discovery'!
SystemOrganization addCategory: #'SqSquare-Server-WebUI-Admin'!
SystemOrganization addCategory: #'SqSquare-Server-WebUI'!

ElCommandServer subclass: #SqAudioChatCommandServer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqSquare-Server-AudioChat'!

!SqAudioChatCommandServer methodsFor: 'initialize-release' stamp: 'minami 5/11/2006 15:03'!
connected: aSocket

	| uid unit existing |

	uid := aSocket receive: 6 timeout: 3.
	uid := SqUid withAll: uid.

	unit := ElAudioChatUnit uid: uid socket: aSocket.
	unit stoppedHandler: [ :u | self unitStopped: u ].
	unit receivedHandler: [ :u :aByteArray | self received: aByteArray from: u uid ]. 

	unitsMutex critical: [
		existing := units at: uid ifAbsent: [].
		existing ifNotNil: [
			[
				existing stoppedHandler: nil; stop.
			] forkAt: Processor systemBackgroundPriority.
		].
		units at: uid put: unit.
	].

	unit start.
! !

!SqAudioChatCommandServer methodsFor: 'running' stamp: 'minami 5/19/2006 14:31'!
restartInvalidConnectionHandler
	connector ifNil: [^ self].
	connector restartIf: [connector hasInvalidListener].! !

SBComponent subclass: #SqBoardOpener
	instanceVariableNames: 'sessionParams sessionKey'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqSquare-Server-WebUI'!

!SqBoardOpener class methodsFor: 'constants' stamp: 'mu 4/18/2006 17:21'!
dummyResourceExtension
	^'.sqsq'! !

!SqBoardOpener methodsFor: 'accessing' stamp: 'mu 4/18/2006 19:15'!
boardId
	^sessionParams second! !

!SqBoardOpener methodsFor: 'private' stamp: 'mu 6/6/2006 14:20'!
commandEnter
	| data |
	data := String
				streamContents: [:str | 
					str nextPutAll: 'sessionKey=' , self sessionKey; cr.
					str nextPutAll: 'boardId=' , self boardId; cr.
					str nextPutAll: 'userId=' , self userId; cr.
					str nextPutAll: 'encodedPassword=' , self encodedPassword; cr.
					str nextPutAll: 'hostname=' , self hostname; cr.
				].
	^ self return: data named: ('Enter', self dummyResourceExtension)! !

!SqBoardOpener methodsFor: 'private' stamp: 'mu 5/24/2006 11:55'!
commandInform: aString
	^self return: ('inform=', aString) named: ('Inform', self dummyResourceExtension)! !

!SqBoardOpener methodsFor: 'accessing' stamp: 'mu 5/24/2006 16:38'!
db
	^SqSquareDB default! !

!SqBoardOpener methodsFor: 'accessing' stamp: 'mu 5/24/2006 11:55'!
dummyResourceExtension
	^self class dummyResourceExtension! !

!SqBoardOpener methodsFor: 'accessing' stamp: 'mu 6/6/2006 14:19'!
encodedPassword
	| user |
	user _ self db userOf: self userId.
	^ (user
		ifNotNil: [user password]) asString! !

!SqBoardOpener methodsFor: 'private' stamp: 'mu 4/19/2006 12:06'!
extractParamsFrom: targetName 
	^ (targetName readStream upTo: $.)
		findTokens: ' '! !

!SqBoardOpener methodsFor: 'accessing' stamp: 'mu 6/6/2006 15:02'!
hostname
	"^self currentUrlHead asUrl authority"

	^SqSettings serverHost! !

!SqBoardOpener methodsFor: 'private' stamp: 'mu 4/18/2006 19:20'!
isValidSessionKey

	| sessionClass  launcherSession |

	self sessionKey ifNil: [^false].

	sessionClass _ self session application sessionClass.
	launcherSession _ sessionClass allInstances detect: [:e | e sessionKey = self sessionKey] ifNone: [].

	^launcherSession notNil! !

!SqBoardOpener methodsFor: 'accessing' stamp: 'mu 4/21/2006 15:49'!
port
	^self currentUrlHead asUrl post asString! !

!SqBoardOpener methodsFor: 'rendering' stamp: 'mu 6/8/2006 14:03'!
renderContentOn: html 
	| resourcePath targetName |
	resourcePath := self session currentRequest url findTokens: '/'.
	targetName := resourcePath last.
	(targetName endsWith: self class dummyResourceExtension)
		ifTrue: [sessionParams := self extractParamsFrom: targetName]
		ifFalse: [^self commandInform: 'Bad request' translated ].
	sessionParams ifNotNil: [^self returnEnterCommand]! !

!SqBoardOpener methodsFor: 'actions' stamp: 'mu 6/8/2006 14:04'!
returnEnterCommand
	self isValidSessionKey ifFalse: [^self commandInform: ('Invalid sessionKey:' translated, self sessionKey)].
	self commandEnter

	! !

!SqBoardOpener methodsFor: 'private' stamp: 'mu 4/18/2006 19:35'!
return: data named: fileName
	self session
		download: data zipped
		mimeType: 'application/x-squeak-project'
		filename: fileName

	! !

!SqBoardOpener methodsFor: 'accessing' stamp: 'mu 4/18/2006 19:20'!
sessionKey
	sessionParams ifNil: [^nil].
	^sessionParams first! !

!SqBoardOpener methodsFor: 'accessing' stamp: 'mu 4/18/2006 19:14'!
sessionParams
	^sessionParams! !

!SqBoardOpener methodsFor: 'accessing' stamp: 'mu 4/18/2006 19:15'!
userId
	^sessionParams third! !

SBComponent subclass: #SqBoardPluginLauncher
	instanceVariableNames: 'board'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqSquare-Server-WebUI'!

!SqBoardPluginLauncher methodsFor: 'accessing' stamp: 'mu 4/18/2006 14:39'!
board
	^board! !

!SqBoardPluginLauncher methodsFor: 'accessing' stamp: 'mu 4/18/2006 18:55'!
boardId
	^ self board id asString! !

!SqBoardPluginLauncher methodsFor: 'accessing' stamp: 'mu 4/18/2006 14:40'!
board: aSqBoard
	board _ aSqBoard! !

!SqBoardPluginLauncher methodsFor: 'private' stamp: 'mu 5/24/2006 12:06'!
buildLocationPath
	| base params |
	base := self session application baseUrl path first.
	params := String
				streamContents: [:str | {self sessionKey. self boardId. self userId}
						do: [:each | str nextPutAll: each]
						separatedBy: [str nextPutAll: '+']].
	^ self currentUrlHead , '/' , base , '/' , SqBoardOpener applicationName asString , '/' , params , SqBoardOpener dummyResourceExtension! !

!SqBoardPluginLauncher methodsFor: 'private' stamp: 'mu 4/18/2006 14:45'!
determineMimeType
	"'application/x-squeak-source' is used for old ns plugin"
	| agent |

	self flag: #needToPushUp.

	agent _ self session currentRequest headerAt: 'user-agent'.
	(agent beginsWith: 'Mozilla/5.0 (Windows;') ifTrue: [^'application/x-squeak-source'].

	^'application/x-squeak-project'! !

!SqBoardPluginLauncher methodsFor: 'rendering' stamp: 'mu 4/18/2006 14:44'!
renderContentOn: html 
	| location |
	super renderContentOn: html.
	html heading: self board name.
	location := self buildLocationPath.
	self squeakProjectObjectTagOn: html location: location! !

!SqBoardPluginLauncher methodsFor: 'accessing' stamp: 'mu 4/18/2006 18:54'!
sessionKey
	^ self session sessionKey! !

!SqBoardPluginLauncher methodsFor: 'private' stamp: 'mu 4/18/2006 18:58'!
squeakProjectObjectTagOn: html location: fileLocationURL

self flag: #needToPushUp.

html objectId: 'SqueakOCX Control' width: '1024' height: '768' align: 'center' border: '2' classid: 'CLSID:2BE9C39E-8386-4435-B337-FCDAD8EAB006' codebase: 'http://www.squeakland.org/installers/SqueakPluginInstaller.exe' do:[

html param: 'src' value: fileLocationURL.
html param: 'update' value: '144'.
html param: 'pluginspage' value: 'http://www.squeakland.org/detect.html'.
html param: 'update_url' value: 'http://www.squeakland.org/update/'.
html param: 'apiSupported' value: 'true'.
html param: 'sessionKey' value: (self sessionKey).
html param: 'boardId' value: (self boardId).
html param: 'userId' value: (self userId).

html attributeAt: 'update' put: '144'.
html attributeAt: 'apiSupported' put: 'true'.
html attributeAt: 'pluginspage' put: 'http://www.squeakland.org/detect.html'.
html attributeAt: 'update_url' put: 'http://www.squeakland.org/update/'.
html attributeAt: 'sessionKey' put: (self session sessionKey).
html attributeAt: 'boardId' put: (self boardId).
html attributeAt: 'userId' put: (self userId).
html embed: fileLocationURL type: self determineMimeType align: 'center' width: '1024' height: '768'.

]! !

!SqBoardPluginLauncher methodsFor: 'accessing' stamp: 'mu 5/24/2006 12:06'!
userId
	self isAuthenticated ifFalse: [^'%3F'].
	^ self session authentication uid asString

! !

SBComponent subclass: #SqWebComponent
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqSquare-Server-WebUI'!

SqWebComponent subclass: #SqBoardRollbackList
	instanceVariableNames: 'board comboBox boardServer'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqSquare-Server-WebUI-Admin'!

!SqBoardRollbackList methodsFor: 'accessing' stamp: 'mu 6/19/2006 14:13'!
board
	^board! !

!SqBoardRollbackList methodsFor: 'accessing' stamp: 'mu 6/19/2006 14:13'!
board: aBoard
	board _ aBoard! !

!SqBoardRollbackList methodsFor: 'builder' stamp: 'mu 6/19/2006 14:46'!
buildComboBox
	comboBox := SBComboBox parent: self.
	comboBox 
		 label: 'snapshots' translated0;
		 id: #snapshotList! !

!SqBoardRollbackList methodsFor: 'accessing' stamp: 'mu 6/19/2006 14:31'!
children
	^{comboBox}! !

!SqBoardRollbackList methodsFor: 'accessing' stamp: 'mu 6/19/2006 14:35'!
comboBox
	comboBox ifNil: [self buildComboBox].
	^comboBox! !

!SqBoardRollbackList methodsFor: 'callback' stamp: 'mu 6/19/2006 15:10'!
comboBoxSelected: item from: aspect

	| ans |
	ans _ self confirm: (('Rollback to {1}??' translated0) format: {item value asString}).
	ans ifFalse: [self answer: nil].

	boardServer ifNotNilDo: [:bs | bs rollbackSnapshotAt: item value].
	

	! !

!SqBoardRollbackList methodsFor: 'rendering' stamp: 'mu 6/19/2006 14:59'!
renderContentOn: html

	html heading: 'Rollback list' translated0.
	
	boardServer _ SqServer default boardServerAt: self board boardId ifAbsent:[].
	boardServer ifNotNilDo: [:bs | self renderRollbackListOn: html].

	html horizontalRule.
	
	! !

!SqBoardRollbackList methodsFor: 'rendering' stamp: 'mu 6/19/2006 14:55'!
renderRollbackListOn: html 
	html text: 'Select snapshot to repair'.
	self comboBox list: boardServer listSnapshots asSortedCollection.

	html render: self comboBox! !

SqWebComponent subclass: #SqIntroduction
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqSquare-Server-WebUI'!

!SqIntroduction methodsFor: 'accessing' stamp: 'mu 4/12/2006 17:22'!
name
	^self pageTitle! !

!SqIntroduction methodsFor: 'accessing' stamp: 'mu 4/12/2006 17:26'!
pageTitle
	"Returns the receivers page title"

	^'Introduction' translated! !

!SqIntroduction methodsFor: 'rendering' stamp: 'mu 6/8/2006 14:41'!
renderContentOn: html
	html heading: 'What''s SqSquare?' translated. 
	html paragraph: 'SqSquare is a collaboration system, which provides broad virtual desktops.' translated.

	html paragraph: 'You can freely draw pictures and write texts on a shared board.' translated.

	html paragraph: 'You can also start voice-chatting with your friends.' translated.

	html horizontalRule.

	self renderMoreOn: html 

	! !

!SqIntroduction methodsFor: 'rendering' stamp: 'mu 6/8/2006 14:44'!
renderMoreOn: html 
	| url |
	url := self session application preferenceAt: #moreInfoUrl.
	url isEmptyOrNil ifTrue: [^ self].
	html
		anchorWithText: url
		do: [html paragraph: 'More' translated , ' >>']! !

!SqWebComponent methodsFor: 'accessing' stamp: 'mu 4/12/2006 12:11'!
db
	^SqSquareDB default! !

Object subclass: #SqBoardServer
	instanceVariableNames: 'eventsHolder broadcaster boardId activeUserSerials'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqSquare-Server'!

!SqBoardServer class methodsFor: 'instance creation' stamp: 'minami 3/29/2006 15:20'!
boardId: boardId 
	^ self new boardId: boardId;
		 yourself! !

!SqBoardServer methodsFor: 'accessing' stamp: 'mu 6/22/2006 16:52'!
activeUserSerials
	activeUserSerials ifNil: [activeUserSerials _ WeakValueDictionary new].
	^activeUserSerials! !

!SqBoardServer methodsFor: 'accessing' stamp: 'mu 6/22/2006 17:26'!
activeUsers
	| ret |
	ret _  self activeUserSerials values
		collect: [:each | each ifNotNil: [self db userProfileAt: each userSerial]]
		thenSelect: [:each | each notNil].
	^ret sort: [:a :b | a < b]! !

!SqBoardServer methodsFor: 'actions-events' stamp: 'mu 6/22/2006 16:53'!
addEvent: aSqEvent
	aSqEvent isDisposable ifTrue: [^self].
	self eventsHolder add: aSqEvent.

	self activeUserSerials at: aSqEvent userSerial put: aSqEvent! !

!SqBoardServer methodsFor: 'accessing' stamp: 'MU 1/20/2006 17:26'!
boardId
	^boardId! !

!SqBoardServer methodsFor: 'accessing' stamp: 'MU 1/20/2006 17:27'!
boardId: anUUID
	boardId _ anUUID! !

!SqBoardServer methodsFor: 'actions-events' stamp: 'mu 4/6/2006 16:23'!
bootstrapEventNumber
	
	| lastNumberInSnap lastNumberInDb |
	lastNumberInSnap _ SqSnapshotReader latestEventNumberOf: self boardId.
	lastNumberInDb _ self db latestEventNumberOf: self boardId.
	self eventsHolder bootstrapEventNumber: (lastNumberInSnap max: lastNumberInDb)! !

!SqBoardServer methodsFor: 'accessing' stamp: 'minami 2/28/2006 17:04'!
broadcaster
	broadcaster
		ifNil: [broadcaster _ SqBroadcaster new].
	^ broadcaster! !

!SqBoardServer methodsFor: 'accessing' stamp: 'minami 2/28/2006 17:06'!
broadcaster: aBroadcaster
	broadcaster _ aBroadcaster! !

!SqBoardServer methodsFor: 'actions' stamp: 'mu 4/28/2006 10:36'!
broadcast: evt 
	self broadcaster
		ifNotNilDo: [:b | 
			b isRunning
				ifFalse: [b start].
			b add: evt]! !

!SqBoardServer methodsFor: 'actions' stamp: 'mu 3/8/2006 17:11'!
close
	self stop.
	self forgetDumpedEvents.
	self initialize! !

!SqBoardServer methodsFor: 'accessing' stamp: 'mu 4/5/2006 16:52'!
currentNumber
	^ self eventsHolder currentNumber! !

!SqBoardServer methodsFor: 'accessing' stamp: 'MU 2/24/2006 12:31'!
db
	^SqSquareDB default! !

!SqBoardServer methodsFor: 'actions-events' stamp: 'mu 4/4/2006 14:56'!
dbStoredEvents
	self db ifNil: [^#()].
	^self db allEventsOf: self boardId! !

!SqBoardServer methodsFor: 'snapshot' stamp: 'mu 6/1/2006 23:31'!
deleteAllSnapshots
	| dir entries |
	dir := SqSettings snapshotDirectory.
	entries _ dir filesMatches: [:each | each name beginsWith: self boardId asString].
	entries do: [:each | dir deleteFileNamed: each name].

	SqSnapshotTaker initialize! !

!SqBoardServer methodsFor: 'actions-events' stamp: 'mu 6/22/2006 16:55'!
dumpUpToEnd
	| memEvents |
	memEvents := self eventsHolder dumpUpToEnd.
	[self db addEvents: memEvents to: self boardId]
		on: SqEventRenumbered
		do: [:ex | 
			self eventsHolder
				bootstrapEventNumber: (memEvents last eventId)].
	^memEvents! !

!SqBoardServer methodsFor: 'actions-events' stamp: 'mu 4/5/2006 16:52'!
events
	self eventsHolder currentNumber <= 0 ifTrue: [^#()].
	^self eventsFrom: 1 to: self eventsHolder currentNumber! !

!SqBoardServer methodsFor: 'actions-events' stamp: 'mu 4/5/2006 16:54'!
eventsFrom: start to: end 
	| firstEventNumber dbEvents memEvents  |
	
	firstEventNumber _ self eventsHolder firstEventNumber.
	firstEventNumber < 0 ifTrue: [^self db eventsFrom: start to: end of: self boardId].
	firstEventNumber <= start ifTrue: [
		| startIndex endIndex |
		startIndex _ start - firstEventNumber + 1.
		endIndex _ startIndex + (end - start).
		^ self eventsHolder eventsFrom: startIndex to: endIndex.
	].

	dbEvents _ self db eventsFrom: start to: (end min: firstEventNumber -1) of: self boardId.
	memEvents _ self eventsHolder eventsFrom: 1 to: ((end - dbEvents size - start + 1) min: self onMemoryEvents size).
	^dbEvents , memEvents! !

!SqBoardServer methodsFor: 'accessing' stamp: 'mu 4/28/2006 14:52'!
eventsHolder
	eventsHolder ifNil: [eventsHolder := SqServerEventRecordHolder new].
	^ eventsHolder! !

!SqBoardServer methodsFor: 'actions-events' stamp: 'mu 3/8/2006 17:13'!
forgetDumpedEvents
	self db removeGroup: (SqBoard id: self boardId).
	! !

!SqBoardServer methodsFor: 'initialize' stamp: 'mu 6/22/2006 16:33'!
initialize
	super initialize.
	eventsHolder _ nil.
	broadcaster _ nil.
	activeUsers _ nil.! !

!SqBoardServer methodsFor: 'statistics' stamp: 'MU 1/20/2006 17:06'!
inspectConnections
	^ SqPassiveConnection incomingConnectionDict inspect! !

!SqBoardServer methodsFor: 'actions-events' stamp: 'mu 4/5/2006 16:55'!
latestEventsFrom: start 
	| firstEventNumber dbEvents memEvents |
	
	firstEventNumber _ self eventsHolder firstEventNumber.
	firstEventNumber < 0 ifTrue: [^self db latestEventsFrom: start of: self boardId].
	firstEventNumber <= start ifTrue: [^self eventsHolder latestEventsFrom: (start - firstEventNumber + 1)].

	dbEvents _ self db eventsFrom: start to: firstEventNumber -1 of: self boardId.
	memEvents _ self eventsHolder latestEventsFrom: 1.
	^dbEvents , memEvents! !

!SqBoardServer methodsFor: 'snapshot' stamp: 'mu 6/19/2006 14:48'!
listSnapshots
	| dir entries |
	dir := SqSettings snapshotDirectory.
	entries _ dir filesMatches: [:each | each name beginsWith: self boardId asString].
	^entries collect: [:each | (DateAndTime fromSeconds: each modificationTime) -> (SqSnapshotReader versionNumberOf: each name)]! !

!SqBoardServer methodsFor: 'actions-events' stamp: 'mu 4/5/2006 16:52'!
onMemoryEvents
	^self eventsHolder events! !

!SqBoardServer methodsFor: 'printing' stamp: 'MU 2/9/2006 14:52'!
printOn: aStream
	aStream nextPutAll: 'a ', self className, '('.
	boardId ifNotNilDo: [:id | aStream nextPutAll: 'boardId: ', id asString].
	aStream nextPutAll: ')'! !

!SqBoardServer methodsFor: 'actions-events' stamp: 'mu 4/5/2006 16:51'!
resetEventRecords
	"reset on memory events only"
	eventsHolder ifNotNilDo: [:rh | rh resetEventRecords]! !

!SqBoardServer methodsFor: 'snapshot' stamp: 'mu 6/19/2006 15:12'!
rollbackSnapshotAt: eventId 
	| boardIdString |
	self eventsHolder dumpUpToEnd.
	self eventsHolder bootstrapEventNumber: eventId.
	self db discardEventsFrom: eventId of: boardId.
	boardIdString _ boardId asString.
	SqSettings snapshotDirectory
		deleteFilesMatches: [:each | (each name beginsWith: boardIdString)
				and: [(SqSnapshotReader versionNumberOf: each name) > eventId]].

	SqSnapshotTaker removeBoardAt: self boardId.
	SqBoardThumbnailCache default removeKey: self boardId.! !

!SqBoardServer methodsFor: 'running' stamp: 'MU 3/1/2006 10:11'!
start
	self stop.
	self broadcaster start.
! !

!SqBoardServer methodsFor: 'running' stamp: 'mu 4/6/2006 12:10'!
stop
	self broadcaster stop.
	self broadcaster: nil.! !

!SqBoardServer methodsFor: 'snapshot' stamp: 'mu 6/14/2006 09:11'!
takeSnapshot
	| events st |
	events := self dumpUpToEnd.
	events isEmptyOrNil
		ifTrue: [^ nil].

	self db setBoardUpdatedTimeAt: boardId. 

	st := SqSnapshotTaker appendAll: events originalBoardId: boardId.
	^ st takeSnapshot! !

!SqBoardServer methodsFor: 'snapshot' stamp: 'mu 6/14/2006 09:11'!
takeSnapshotFromFirst
	| events st |
	events := self events.
	events isEmptyOrNil
		ifTrue: [^ nil].

	self db setBoardUpdatedTimeAt: boardId. 

	st := SqSnapshotTaker withAll: events originalBoardId: boardId.
	^ st takeSnapshot! !

Object subclass: #SqManager
	instanceVariableNames: 'db'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqSquare-Server-Service'!

SqManager subclass: #SqBoardManager
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqSquare-Server-Service'!

!SqBoardManager methodsFor: 'accessing' stamp: 'mu 4/26/2006 14:38'!
boards
	^self db groups! !

!SqBoardManager methodsFor: 'actions' stamp: 'mu 5/9/2006 12:32'!
createNewBoard: boardName 
	| bd |
	self db
		ifNil: [^ nil].
	bd := SqBoard id: SqUid new name: boardName.
	self db registerGroup: bd.
	^ bd! !

!SqBoardManager methodsFor: 'actions' stamp: 'mu 6/22/2006 17:06'!
getActiveUsersOf: boardId
	| boardServer |
	boardServer _ SqServer default boardServerAt: boardId ifAbsent: [^#()].
	^boardServer activeUsers! !

!SqBoardManager methodsFor: 'actions' stamp: 'MU 2/22/2006 15:16'!
getAllEventsOf: boardId
	| boardServer |
	boardServer _ SqServer default boardServerAt: boardId ifAbsent: [^#()].
	^boardServer events! !

!SqBoardManager methodsFor: 'actions' stamp: 'MU 2/22/2006 15:15'!
getEventsOf: boardId from: startIndex to: endIndex
	| boardServer |
	boardServer _ SqServer default boardServerAt: boardId ifAbsent: [^#()].
	^boardServer eventsFrom: startIndex to: endIndex  ! !

!SqBoardManager methodsFor: 'actions' stamp: 'MU 2/22/2006 15:15'!
getLatestEventsOf: boardId from: startIndex
	| boardServer |
	boardServer _ SqServer default boardServerAt: boardId ifAbsent: [^#()].
	^boardServer latestEventsFrom: startIndex! !

!SqBoardManager methodsFor: 'actions' stamp: 'mu 6/5/2006 14:23'!
getLatestSnapshotOf: boardId 
	^ (SqSnapshotTaker boardAt: boardId)
		ifNil: [SqSnapshotReader readLatestSnapshotOf: boardId]
		ifNotNilDo: [:b | {boardId. SqSnapshotReader latestEventNumberOf: boardId. b asSnapshotData}]! !

!SqBoardManager methodsFor: 'actions' stamp: 'MU 2/22/2006 15:13'!
removeBoard: board 
	self db
		ifNil: [^ nil].
	^ self db removeGroup: board! !

SqManager subclass: #SqLoginManager
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqSquare-Server-Service'!

!SqLoginManager methodsFor: 'actions' stamp: 'mu 5/24/2006 17:05'!
login: userId encodedPassword: password 
	| verified |
	self db
		ifNil: [^ false].
	verified _ self db verifyEncodedPassword: password forUser: userId.
	^ verified
		ifTrue: [| user serial | 
			user _ self db userOf: userId.
			user ifNil: [^ nil].
			serial _ user serialNumber.
			SqAuthentication userId: userId serialNumber: serial]! !

!SqLoginManager methodsFor: 'actions' stamp: 'mu 5/22/2006 16:42'!
login: userId password: password 
	| verified |
	self db
		ifNil: [^ false].
	verified _ self db verifyPassword: password forUser: userId.
	^ verified
		ifTrue: [| user serial | 
			user _ self db userOf: userId.
			user ifNil: [^ nil].
			serial _ user serialNumber.
			SqAuthentication userId: userId serialNumber: serial]! !

!SqLoginManager methodsFor: 'actions' stamp: 'mu 5/22/2006 15:28'!
logout: authentication 
	"do we need this?"
	SqServer closeAllConnectionsByUser: authentication serialNumber.
	^true! !

!SqLoginManager methodsFor: 'registering' stamp: 'MU 2/22/2006 15:09'!
register: userId password: password 
	| user |
	self db
		ifNil: [^ nil].
	user := SqUser id: userId password: password.
	self db registerUser: user.
	^ user! !

!SqLoginManager methodsFor: 'registering' stamp: 'MU 2/22/2006 15:07'!
unregister: userId
	
	^self db ifNotNilDo: [:d | d removeUser: (SqUser id: userId )]! !

!SqLoginManager methodsFor: 'accessing' stamp: 'MU 2/22/2006 15:02'!
users
	self db
		ifNil: [^ #()].
	^ self db users! !

!SqManager methodsFor: 'accessing' stamp: 'MU 2/22/2006 14:57'!
db
	db ifNil: [db _Smalltalk at: #SqSquareDB ifPresent:[:dbClass | dbClass default]].
	^db! !

!SqManager methodsFor: 'accessing' stamp: 'MU 2/22/2006 14:57'!
db: object
	db _ object! !

!SqManager methodsFor: 'initialize-release' stamp: 'MU 2/22/2006 14:57'!
initialize
	db _ nil! !

SqManager subclass: #SqUserManager
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqSquare-Server-Service'!

!SqUserManager methodsFor: 'registering' stamp: 'MU 2/22/2006 11:56'!
register: aSqUser
	self db registerUser: aSqUser.
	^ aSqUser! !

!SqUserManager methodsFor: 'registering' stamp: 'MU 2/21/2006 16:10'!
unregister: aSqUser
	self db removeUser: aSqUser.
	^ aSqUser! !

!SqUserManager methodsFor: 'accessing' stamp: 'mu 6/22/2006 17:06'!
userProfileAt: serial
	^self db userProfileAt: serial! !

!SqUserManager methodsFor: 'accessing' stamp: 'mu 4/26/2006 14:42'!
userProfileOf: uid
	^self db userProfileOf: uid! !

!SqUserManager methodsFor: 'accessing' stamp: 'MU 2/21/2006 15:52'!
users
	^self db users! !

Object subclass: #SqServer
	instanceVariableNames: 'boardServers'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqSquare-Server'!

SqServer class
	instanceVariableNames: 'default'!

!SqServer class methodsFor: 'connection management' stamp: 'mu 5/22/2006 15:18'!
activeUsers
	| users |
	users _ Set new.
	self allConnections do: [:each | users add: each accessId userSerial].
	^users ! !

!SqServer class methodsFor: 'connection management' stamp: 'minami 1/11/2006 15:41'!
allConnections
	^SqPassiveConnection connections! !

!SqServer class methodsFor: 'utilities' stamp: 'mu 4/6/2006 12:27'!
bootstrapAll
	^self default bootstrapAll! !

!SqServer class methodsFor: 'connection management' stamp: 'MU 2/22/2006 17:12'!
closeAllConnectionsByBoard: boardId 
	self
		closeAllConnectionsMatches: [:each | each accessId boardId = boardId]! !

!SqServer class methodsFor: 'connection management' stamp: 'mu 5/22/2006 15:28'!
closeAllConnectionsByUser: userSerialNumber 
	self
		closeAllConnectionsMatches: [:each | each accessId userSerial = userSerialNumber]! !

!SqServer class methodsFor: 'connection management' stamp: 'MU 2/22/2006 17:12'!
closeAllConnectionsMatches: aBlock
	| matched |
	matched _ self allConnections select: [:each | aBlock value: each].
	matched do: [:each | each close]! !

!SqServer class methodsFor: 'instance creation' stamp: 'minami 1/11/2006 13:53'!
default
	default ifNil: [default := super new].
	^ default! !

!SqServer class methodsFor: 'utilities' stamp: 'MU 2/22/2006 17:05'!
deleteAll
	^self default deleteAll! !

!SqServer class methodsFor: 'utilities' stamp: 'MU 2/22/2006 17:05'!
forgetAll
	^self default forgetAll! !

!SqServer class methodsFor: 'class initialization' stamp: 'minami 1/11/2006 13:54'!
initialize
	default := nil! !

!SqServer class methodsFor: 'running' stamp: 'mu 3/2/2006 12:07'!
resetStart
	self stop.
	self deleteAll.
	self initialize.
	self start.
	self default inspectAll.
	
	^ self default ! !

!SqServer class methodsFor: 'utilities' stamp: 'MU 2/22/2006 17:05'!
snapshotAll
	^self default snapshotAll! !

!SqServer class methodsFor: 'utilities' stamp: 'mu 4/7/2006 15:33'!
snapshotFromFirstAll
	^self default snapshotFromFirstAll! !

!SqServer class methodsFor: 'running' stamp: 'mu 3/2/2006 12:06'!
start
	self default start.
	^ self default ! !

!SqServer class methodsFor: 'running' stamp: 'mu 3/2/2006 12:06'!
stop
	self default stop.
	^ self default ! !

!SqServer methodsFor: 'accessing' stamp: 'minami 1/27/2006 15:16'!
boardServerAt: boardId 
	^ self boardServers
		at: boardId
		ifAbsent: []! !

!SqServer methodsFor: 'accessing' stamp: 'minami 1/27/2006 14:31'!
boardServerAt: boardId ifAbsentPut: aBlock
	^self boardServers 
		at: boardId 
		ifAbsentPut: aBlock ! !

!SqServer methodsFor: 'accessing' stamp: 'MU 2/6/2006 14:47'!
boardServerAt: boardId ifAbsent: aBlock
	^self boardServers 
		at: boardId 
		ifAbsent: aBlock ! !

!SqServer methodsFor: 'accessing' stamp: 'MU 1/20/2006 17:18'!
boardServers
	boardServers ifNil: [boardServers _ Dictionary new].
	^ boardServers! !

!SqServer methodsFor: 'actions' stamp: 'mu 4/6/2006 12:26'!
bootstrapAll
	
	self boardServers do: [:e | e bootstrapEventNumber]
	! !

!SqServer methodsFor: 'debugging' stamp: 'MU 2/22/2006 17:19'!
connections
	^self class allConnections! !

!SqServer methodsFor: 'actions' stamp: 'MU 2/17/2006 14:05'!
deleteAll
	self boardServers do: [:e | e deleteAllSnapshots]
	! !

!SqServer methodsFor: 'actions' stamp: 'MU 2/24/2006 13:00'!
forgetAll
	self boardServers
		do: [:e | e resetEventRecords].

	self boardServers
		do: [:e | SqSquareDB default removeAllEventsFrom: e boardId]! !

!SqServer methodsFor: 'initialize' stamp: 'MU 1/20/2006 17:15'!
initialize
	super initialize.
! !

!SqServer methodsFor: 'debugging' stamp: 'MU 2/6/2006 15:54'!
inspectAll
	^ (Dictionary newFrom: 
		{ 
		  #services -> self services. 
		  #connections -> self connections.
		  #boardServers -> self boardServers
		}
	) inspect! !

!SqServer methodsFor: 'debugging' stamp: 'MU 2/6/2006 15:49'!
inspectBoardServers
	self boardServers inspect! !

!SqServer methodsFor: 'debugging' stamp: 'MU 2/6/2006 15:53'!
inspectConnections
	^self connections inspect! !

!SqServer methodsFor: 'debugging' stamp: 'MU 2/6/2006 15:53'!
inspectServices
	^ self services inspect! !

!SqServer methodsFor: 'running' stamp: 'mu 3/9/2006 16:59'!
isRunning
	^SqPassiveConnection
		isRunningAt: SqSettings eventListenPort
		! !

!SqServer methodsFor: 'accessing' stamp: 'mu 5/26/2006 15:50'!
removeBoardServerAt: boardId 
	| bs |
	bs _ self boardServerAt: boardId ifAbsent: [^nil].
	bs close.
	self boardServers removeKey: boardId ! !

!SqServer methodsFor: 'debugging' stamp: 'MU 2/6/2006 15:53'!
services
	^ {SqPassiveConnection serviceForReceiveDict. SqPassiveConnection serviceForReturnDict}! !

!SqServer methodsFor: 'actions' stamp: 'mu 4/7/2006 14:47'!
snapshotAll
	self boardServers do: [:e | e takeSnapshot]
	! !

!SqServer methodsFor: 'actions' stamp: 'mu 4/7/2006 15:32'!
snapshotFromFirstAll
	self boardServers do: [:e | e takeSnapshotFromFirst]
	! !

!SqServer methodsFor: 'running' stamp: 'mu 3/9/2006 16:59'!
start
	self isRunning ifTrue: [^self].
	self startListening.
! !

!SqServer methodsFor: 'running' stamp: 'minami 1/27/2006 01:32'!
startListening
	SqPassiveConnection
		listenOn: SqSettings eventListenPort
		acceptorClass: SqBoardServerAcceptor! !

!SqServer methodsFor: 'running' stamp: 'mu 3/8/2006 11:25'!
stop
	SqPassiveConnection stopListening.

	self boardServers do: [:each | each stop].! !

Object subclass: #SqServicePublisher
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqSquare-Server-Discovery'!

SqServicePublisher class
	instanceVariableNames: 'boardServiceEntry chatServiceEntry'!

!SqServicePublisher class methodsFor: 'accessing' stamp: 'mu 3/27/2006 11:13'!
boardServiceEntry
	boardServiceEntry ifNil: [self prepareBoardServiceEntry].
	^boardServiceEntry! !

!SqServicePublisher class methodsFor: 'accessing' stamp: 'mu 3/27/2006 11:13'!
chatServiceEntry
	chatServiceEntry ifNil: [self prepareChatServiceEntry].
	^chatServiceEntry! !

!SqServicePublisher class methodsFor: 'class initialization' stamp: 'mu 3/27/2006 11:29'!
initialize
	"SqServicePublisher initialize"
	Smalltalk addToStartUpList: self.
	Smalltalk addToShutDownList: self.

	self initServiceEntries.

	self startUp! !

!SqServicePublisher class methodsFor: 'class initialization' stamp: 'mu 3/27/2006 11:09'!
initServiceEntries
	"SqServicePublisher initServiceEntries"
	boardServiceEntry _ chatServiceEntry _ nil.

	! !

!SqServicePublisher class methodsFor: 'accessing' stamp: 'mu 3/31/2006 14:14'!
localAddressString
	
	^ SqUtil localAddressStringAskIfDetectionFailed! !

!SqServicePublisher class methodsFor: 'class initialization' stamp: 'mu 3/11/2006 19:27'!
obsolete
	Smalltalk removeFromStartUpList: self.
	Smalltalk removeFromShutDownList: self! !

!SqServicePublisher class methodsFor: 'private' stamp: 'mu 3/31/2006 14:03'!
prepareBoardServiceEntry
	boardServiceEntry := DsServiceEntry
				type: self serviceType
				version: ''
				name: 'BoardService'
				host: self localAddressString
				port: SqSettings eventListenPort
				maxAge: 10.
	^ boardServiceEntry! !

!SqServicePublisher class methodsFor: 'private' stamp: 'mu 3/31/2006 14:03'!
prepareChatServiceEntry
	chatServiceEntry := DsServiceEntry
				type: self serviceType
				version: ''
				name: 'ChatService'
				host: self localAddressString
				port: SqSettings audioChatPort
				maxAge: 10.
	^ chatServiceEntry! !

!SqServicePublisher class methodsFor: 'constants' stamp: 'mu 3/27/2006 10:26'!
serviceType
	^'SqSquare'! !

!SqServicePublisher class methodsFor: 'start up/shut down' stamp: 'mu 3/11/2006 23:57'!
shutDown: quitting

	quitting ifTrue: [self stopPublish]! !

!SqServicePublisher class methodsFor: 'actions' stamp: 'mu 3/27/2006 15:16'!
startPublish
	"SqServicePublisher startPublish"
	(Smalltalk
		includesKey: #SqServer)
		ifTrue: [DsPublisher defaultForBroadcast
				registerService: self prepareBoardServiceEntry
				while: [SqServer default isRunning]].
	(Smalltalk
		includesKey: #SqAudioChatServerFacade)
		ifTrue: [DsPublisher defaultForBroadcast
				registerService: self prepareChatServiceEntry
				while: [SqAudioChatServerFacade default isRunning]]! !

!SqServicePublisher class methodsFor: 'start up/shut down' stamp: 'mu 3/27/2006 15:10'!
startUp: resuming

	self startPublish! !

!SqServicePublisher class methodsFor: 'actions' stamp: 'mu 3/27/2006 11:39'!
stopPublish
	"SqServicePublisher stopPublish"
	
	DsPublisher defaultForBroadcast
		removeService: self boardServiceEntry.

	DsPublisher defaultForBroadcast
		removeService: self chatServiceEntry.

	self initServiceEntries! !

Object subclass: #SqSnapshotReader
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqSquare-Server-Snapshot'!

!SqSnapshotReader class methodsFor: 'actions' stamp: 'mu 6/19/2006 14:42'!
latestEventNumberOf: boardId
	"SqSnapshotReader latestEventNumberOf: '50DFB1655B98'"
	| entry |
	entry _ SqSettings snapshotDirectory latestFileMatches: [:e | e name beginsWith: boardId asString].
	entry ifNil: [^0].
	^self versionNumberOf: entry name! !

!SqSnapshotReader class methodsFor: 'actions' stamp: 'mu 6/6/2006 12:58'!
latestFormOf: boardId
	"SqSnapshotReader latestFormOf: '50DFB1655B98'"
	| snap player form |
	snap _ self readLatestSnapshotOf: boardId.
	snap ifNil: [^nil].
	player _ snap last asSnappedPlayer.
	form _ player makeIcon: ((player extent min: (800@600)) // 5).
	^form
	! !

!SqSnapshotReader class methodsFor: 'actions' stamp: 'mu 6/2/2006 09:51'!
latestPNGFileOf: boardId
	"[SqSnapshotReader latestPNGFileOf: '50DFB1655B98'] timeToRun"
	| form |
	form _ self latestFormOf: boardId.
	^SqUtil thumbnailPNGFileFromForm: form
	! !

!SqSnapshotReader class methodsFor: 'actions' stamp: 'MU 2/8/2006 10:44'!
loadLatestSnapshotOf: boardId
	"SqSnapshotReader loadLatestSnapshotOf: '6018c1ff-8b9c-b140-a2ed-d62f289f8dc5'"
	| entry |
	entry _ SqSettings snapshotDirectory latestFileMatches: [:e | e name beginsWith: boardId asString].
	entry ifNil: [^nil].
	^self loadSnapshotFileNamed: entry name! !

!SqSnapshotReader class methodsFor: 'actions' stamp: 'MU 2/7/2006 18:31'!
loadSnapshotFileNamed: localFileName 
	"SqSnapshotReader loadSnapshotFileNamed:
	'6018c1ff-8b9c-b140-a2ed-d62f289f8dc5@3316698773' "
	| loaded fileStr |
	[fileStr := SqSettings snapshotDirectory readOnlyFileNamed: localFileName.
	fileStr binary.
	loaded := self loadSnapshotFrom: fileStr]
		ensure: [fileStr
				ifNotNilDo: [:str | str close]].
	^ loaded! !

!SqSnapshotReader class methodsFor: 'actions' stamp: 'mu 5/24/2006 14:12'!
loadSnapshotFrom: fullSnapshotStream
	| snapshot board |
	snapshot _ self readSnapshotDataFrom: fullSnapshotStream.
	board _ CPlayer readSnapshotByteArray: snapshot last.
	board isHeadless: false.
	^board! !

!SqSnapshotReader class methodsFor: 'actions' stamp: 'mu 5/24/2006 14:13'!
readLatestSnapshotOf: boardId
	"SqSnapshotReader readLatestSnapshotOf: '50DFB1655B98'"
	| entry |
	entry _ SqSettings snapshotDirectory latestFileMatches: [:e | e name beginsWith: boardId asString].
	entry ifNil: [^nil].
	^self readSnapshotDataFileNamed: entry name! !

!SqSnapshotReader class methodsFor: 'actions' stamp: 'MU 2/7/2006 18:31'!
readSnapshotDataFileNamed: localFileName
	| loaded fileStr |
	[fileStr _ SqSettings snapshotDirectory readOnlyFileNamed: localFileName.
	fileStr binary.
	loaded := self readSnapshotDataFrom: fileStr]
		ensure: [fileStr
				ifNotNilDo: [:str | str close]].
	^ loaded
! !

!SqSnapshotReader class methodsFor: 'actions' stamp: 'MU 2/6/2006 16:51'!
readSnapshotDataFrom: fullSnapShotStream
	| bytes snapshot |
	bytes _ fullSnapShotStream binary contents.
	snapshot _ RmtStreamer unstream: bytes by: #DataStream.
	^snapshot! !

!SqSnapshotReader class methodsFor: 'utilities' stamp: 'mu 6/19/2006 14:42'!
versionNumberOf: snapshotString 
	^ (snapshotString findTokens: '@') last asNumber! !

Object subclass: #SqSnapshotTaker
	instanceVariableNames: 'events board originatorId'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqSquare-Server-Snapshot'!

SqSnapshotTaker class
	instanceVariableNames: 'boards'!

!SqSnapshotTaker class methodsFor: 'instance creation' stamp: 'mu 6/13/2006 21:25'!
appendAll: events originalBoardId: boardId 
	| board inst |
	board _ self boards
				at: boardId
				ifAbsentPut: [| snappedBoard | 
					snappedBoard _ SqSnapshotReader loadLatestSnapshotOf: boardId.
					snappedBoard
						ifNil: [snappedBoard _ SqEventRecordBoard new].
					snappedBoard isHeadless: true.
					snappedBoard].
	inst _ self new board: board.
	inst events: events.
	inst originatorId: boardId.
	^ inst! !

!SqSnapshotTaker class methodsFor: 'accessing' stamp: 'mu 6/5/2006 14:05'!
boardAt: boardId
	^self boards at: boardId ifAbsent: []! !

!SqSnapshotTaker class methodsFor: 'accessing' stamp: 'mu 6/5/2006 14:06'!
boardAt: boardId ifAbsent: aBlock
	^self boards at: boardId ifAbsent: aBlock! !

!SqSnapshotTaker class methodsFor: 'accessing' stamp: 'mu 6/2/2006 16:11'!
boards
	boards ifNil: [boards _ Dictionary new].
	^boards! !

!SqSnapshotTaker class methodsFor: 'accessing' stamp: 'mu 6/1/2006 23:03'!
boards: aDictionary
	boards _ aDictionary! !

!SqSnapshotTaker class methodsFor: 'class initialization' stamp: 'mu 6/1/2006 23:02'!
initialize
	boards _ nil! !

!SqSnapshotTaker class methodsFor: 'instance creation' stamp: 'mu 6/2/2006 17:01'!
on: aBoard 
	^ self new board: aBoard;
		 originatorId: aBoard boardId;
		 yourself! !

!SqSnapshotTaker class methodsFor: 'accessing' stamp: 'mu 6/19/2006 15:04'!
removeBoardAt: boardId
	^self boards removeKey: boardId ifAbsent: []! !

!SqSnapshotTaker class methodsFor: 'instance creation' stamp: 'MU 1/20/2006 13:57'!
withAll: events
	^self new events: events! !

!SqSnapshotTaker class methodsFor: 'instance creation' stamp: 'MU 2/6/2006 15:03'!
withAll: events originalBoardId: boardId 
	^ self new events: events;
		 originatorId: boardId! !

!SqSnapshotTaker methodsFor: 'accessing' stamp: 'MU 2/6/2006 17:06'!
board
	board
		ifNil: [board := SqEventRecordBoard new isHeadless: true;
						 yourself].
	^ board! !

!SqSnapshotTaker methodsFor: 'accessing' stamp: 'MU 1/20/2006 13:56'!
board: aBoard
	board _ aBoard! !

!SqSnapshotTaker methodsFor: 'actions' stamp: 'MU 2/7/2006 17:57'!
createSnapshot
	self events ifEmpty: [^nil].
	^ {self originatorId. self events last eventId. self rawSnapshot}! !

!SqSnapshotTaker methodsFor: 'actions' stamp: 'mu 4/7/2006 15:06'!
deleteOldSnapshot
	| entry |
	entry := self snapshotDirectory
				oldestFileMatches: [:e | e name beginsWith: originatorId asString].
	entry
		ifNil: [^ nil].
	self snapshotDirectory deleteFileNamed: entry name! !

!SqSnapshotTaker methodsFor: 'accessing' stamp: 'MU 1/20/2006 13:58'!
events
	events ifNil: [events _ OrderedCollection new: 1000].
	^events! !

!SqSnapshotTaker methodsFor: 'accessing' stamp: 'MU 1/20/2006 13:56'!
events: aCollection
	events _ aCollection! !

!SqSnapshotTaker methodsFor: 'accessing' stamp: 'MU 2/13/2006 14:56'!
originatorId
	^originatorId! !

!SqSnapshotTaker methodsFor: 'accessing' stamp: 'MU 1/20/2006 14:02'!
originatorId: id
	originatorId _ id! !

!SqSnapshotTaker methodsFor: 'actions' stamp: 'mu 4/27/2006 16:32'!
rawSnapshot
	self board headlessBoardId: self originatorId.
	self events do: [:each |
		each isDisposable ifFalse: [each execute: self board].
	].
	^self board asSnapshotData! !

!SqSnapshotTaker methodsFor: 'actions' stamp: 'mu 4/7/2006 15:02'!
saveSnapshot
	| dir snapshot bytes fStream |
	dir := self snapshotDirectory.
	snapshot := self createSnapshot.
	snapshot
		ifNotNilDo: [:sn | 
			bytes := RmtStreamer stream: sn.
			fStream := dir forceNewFileNamed: sn first asString , '@' , sn second asString.
			fStream binary.
			[fStream nextPutAll: bytes]
				ensure: [fStream close]]! !

!SqSnapshotTaker methodsFor: 'accessing' stamp: 'mu 4/7/2006 15:01'!
snapshotDirectory
	^SqSettings snapshotDirectory! !

!SqSnapshotTaker methodsFor: 'actions' stamp: 'mu 4/7/2006 15:11'!
takeSnapshot
	self saveSnapshot.
	(self snapshotDirectory
			filesMatches: [:e | e name beginsWith: originatorId asString]) size > SqSettings oldSnapshotsNumber
		ifTrue: [self deleteOldSnapshot]! !

!SqUser methodsFor: '*SqSquare-Server' stamp: 'mu 6/8/2006 15:00'!
addAttributesTo: metaobject

	super addAttributesTo: metaobject.

	metaobject
		passwordAttribute: #password
		do: [:attribute | attribute label: 'password' translated;
				 maxLength: 10;
				 addRequiredRule].

	metaobject dateAttribute: #birthday do: [ :attribute | 
		attribute
			label: 'birthday' translated].

	metaobject textAttribute: #age do: [ :attribute | 
		attribute
			label: 'age' translated;
			readOnly ].

	metaobject booleanAttribute: #male do: [ :attribute | 
		attribute
			label: 'male' translated;
			trueItemString: 'yes' translated falseItemString: 'no' translated ].

	metaobject booleanAttribute: #isAdmin do: [ :attribute | 
		attribute
			label: 'administrator' translated;
			trueItemString: 'yes' translated falseItemString: 'no' translated ].

	metaobject textAttribute: #updatedTime do: [ :attribute | 
		attribute
			label: 'last update' translated;
			readOnly ].

	
	^metaobject! !

!SqUser methodsFor: '*SqSquare-Server' stamp: 'mu 4/14/2006 15:40'!
removeFromDb
	self db removeUser: self! !

!SqUser methodsFor: '*SqSquare-Server' stamp: 'mu 4/14/2006 15:40'!
updateToDb
	self db registerUser: self! !

SBUserManagement subclass: #SqUserManagement
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqSquare-Server-WebUI-Admin'!

!SqUserManagement methodsFor: 'factory' stamp: 'mu 6/8/2006 14:07'!
createTableViewer
	tableViewer := super createTableViewer.
	tableViewer
		anchorMatchBlock: [:nm | nm = 'id' translated].
	^ tableViewer! !

!SqUserManagement methodsFor: 'factory' stamp: 'mu 4/13/2006 12:20'!
db
	^SqSquareDB default! !

!SqUserManagement methodsFor: 'factory' stamp: 'mu 4/21/2006 14:47'!
domainClass
	^SqUser! !

ElAudioChatServer subclass: #SqAudioChatServer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqSquare-Server-AudioChat'!

!SqAudioChatServer methodsFor: 'private' stamp: 'minami 5/11/2006 15:21'!
connected: aSocket

	| uid unit existing |

	self log: 'Unidentified Connection (' , aSocket remotePort asString , ') Established.'.

	[
		uid := aSocket receive: 6 timeout: 3.
		uid := SqUid withAll: uid.
	] on: Error do: [ :err |
		self log: err description.
		err pass.
	].
	self log: 'Connection (' , aSocket remotePort asString , ') Identified UID:' , uid asString , '.'.

	unit := ElAudioChatUnit uid: uid socket: aSocket.
	unit stoppedHandler: [ :u |
		self log: 'Connection Closed UID: ' , u uid asString , '.'.
		self unitStopped: u.
	].
	unit receivedHandler: [ :u :aByteArray |
		self received: aByteArray from: u uid.
	].

	unitsMutex critical: [
		existing := units at: uid ifAbsent: [].
		existing ifNotNil: [
			self log: 'Existing Connection Closed UID:' , uid asString , '.'.
			[ " AudioChatUnit>>stop is protected by unitsMutex "
				existing stoppedHandler: nil; stop.
			] forkAt: Processor systemBackgroundPriority.
		].
		units at: uid put: unit.
		mixer register: uid.
	].

	unit start.
! !

!SqAudioChatServer methodsFor: 'running' stamp: 'minami 5/19/2006 14:32'!
restartInvalidConnectionHandler
	connectionHandler ifNil: [^ self].
	connectionHandler restartIf: [connectionHandler hasInvalidListener].! !

SqEventRecordHolder subclass: #SqServerEventRecordHolder
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqSquare-Server'!

!SqServerEventRecordHolder methodsFor: 'actions' stamp: 'mu 4/28/2006 14:49'!
bootstrapEventNumber: lastEventNumberOnBoot

	self currentNumber: lastEventNumberOnBoot.
	SqUtil renumber: self events from: lastEventNumberOnBoot 

	! !

!SqServerEventRecordHolder methodsFor: 'actions' stamp: 'mu 4/28/2006 14:50'!
dumpUpToEnd
	| oldEvents |
	oldEvents := nil.
	mutex
		critical: [oldEvents := events.
			self initEventRecords].
	^ oldEvents! !

!SqServerEventRecordHolder methodsFor: 'actions' stamp: 'mu 4/28/2006 14:50'!
eventsFrom: start to: end 
	^ [self events copyFrom: start to: end]
		on: Error
		do: [:ex | ex
				resignalAs: (SqOutOfRange
						from: start
						to: end
						actualFrom: 1
						to: self events size)]! !

!SqServerEventRecordHolder methodsFor: 'accessing' stamp: 'mu 4/28/2006 14:51'!
firstEventNumber
	self events ifEmpty: [^-1. "I have no events"].
	^self events first eventId! !

!SqServerEventRecordHolder methodsFor: 'actions' stamp: 'mu 4/28/2006 14:50'!
latestEventsFrom: start
	^self eventsFrom: start to: self events size ! !

!SqBoard methodsFor: '*SqSquare-Server' stamp: 'mu 6/22/2006 17:17'!
activeUsers
	^(SqServer default boardServerAt: self boardId ifAbsent: [^#()]) activeUsers! !

!SqBoard methodsFor: '*SqSquare-Server' stamp: 'mu 6/22/2006 17:24'!
addActiveUsersAttributesTo: metaobject 
	metaobject
		multipleRelationshipAttribute: #activeUsers
		do: [:attribute | 
			attribute readOnly.
			attribute label: 'Active Users' translated0;
				relationshipTo: [self activeUsers]
				formatWith: [:value | value asString , ' ']].
	^ metaobject! !

!SqBoard methodsFor: '*SqSquare-Server' stamp: 'mu 6/22/2006 17:20'!
addAttributesTo: metaobject 
	super addDefaultAttributesTo: metaobject.
	(metaobject attributeOf: #id) readOnly.
	(metaobject attributeOf: #name)
		maxLength: 40.
	self addDescriptionAttributesTo: metaobject.
	self addMembersAttributesTo: metaobject.
	self addActiveUsersAttributesTo: metaobject .
	
	! !

!SqBoard methodsFor: '*SqSquare-Server' stamp: 'mu 6/8/2006 14:00'!
addDescriptionAttributesTo: metaobject 
	metaobject
		textAttribute: #description
		do: [:attribute | attribute label: 'description' translated].
	^metaobject! !

!SqBoard methodsFor: '*SqSquare-Server' stamp: 'mu 5/24/2006 14:35'!
releaseForStore
	super releaseForStore.
	thumbnail _ nil
	! !

!SqBoard methodsFor: '*SqSquare-Server' stamp: 'mu 6/22/2006 17:10'!
thumbnail
	^SqBoardThumbnailCache default at: self boardId.
	! !

!SqBoard methodsFor: '*SqSquare-Server' stamp: 'mu 6/22/2006 17:13'!
thumbnail: aWaFile
	"Currently, only dummy"! !

ApplicationService subclass: #SqBroadcaster
	instanceVariableNames: 'eventQueue'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqSquare-Server'!

!SqBroadcaster class methodsFor: 'accessing' stamp: 'MU 3/1/2006 10:10'!
defaultPriority

	^Processor userInterruptPriority ! !

!SqBroadcaster methodsFor: 'adding' stamp: 'minami 2/28/2006 16:49'!
add: aEvent
	self eventQueue nextPut: aEvent! !

!SqBroadcaster methodsFor: 'actions' stamp: 'mu 4/27/2006 11:14'!
broadcast
	| event |
	event := self eventQueue next.
	event ifNil: [^self].
	SqServer allConnections
		do: [:con | 
			self eventQueue size = 0
				ifTrue: [con send: event. self flag: #TODO. "Can we check accessId here?"]
				ifFalse: [con deferSend: event].
			Processor yield]! !

!SqBroadcaster methodsFor: 'accessing' stamp: 'mu 4/27/2006 11:14'!
eventQueue
	^ eventQueue! !

!SqBroadcaster methodsFor: 'initialize-release' stamp: 'mu 3/7/2006 17:13'!
initialize
	super initialize.

	eventQueue := SharedQueue2 new: 100.! !

!SqBroadcaster methodsFor: 'registration' stamp: 'mu 4/6/2006 12:10'!
register

	"do nothing"! !

!SqBroadcaster methodsFor: 'initialize-release' stamp: 'mu 4/27/2006 11:16'!
release
	self unregister! !

!SqBroadcaster methodsFor: 'initialize-release' stamp: 'mu 4/27/2006 11:14'!
releaseQueue
	eventQueue ifNotNilDo: [:q | q release].
	! !

!SqBroadcaster methodsFor: 'running' stamp: 'minami 2/28/2006 16:29'!
runWhile: aBlock
	[aBlock value] 
		whileTrue: [self broadcast]! !

!SqBroadcaster methodsFor: 'running' stamp: 'mu 4/27/2006 11:15'!
stop
	super stop.
	self releaseQueue.

	self eventQueue nextPut: nil! !

TestCase subclass: #SqDbTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqSquare-Server-Storage-Tests'!

!SqDbTest methodsFor: 'testing' stamp: 'mu 5/9/2006 12:32'!
testAddMembersToGroup
	"SqDbTest debug: #testAddMembersToGroup"
	| db board1 member1 member2 member3 |
	db := SqSquareDB default.
	board1 := SqBoard id: SqUid new name: 'NewBoard1'.
	db registerGroup: board1.
	member1 := SqUser id: 'NewUser1' password: 'auau'.
	member2 := SqUser id: 'NewUser2' password: 'auau'.
	member3 := SqUser id: 'NewUser3' password: 'auau'.

	db addMembers: {member1. member2} to: board1.

	self
		should: [(db membersOf: board1)
				includes: member1].	
	self
		should: [(db membersOf: board1)
				includes: member2].
	self
		shouldnt: [(db membersOf: board1)
				includes: member3].

	db removeGroup: board1! !

!SqDbTest methodsFor: 'testing' stamp: 'mu 5/9/2006 12:31'!
testAddMemberToGroup
	"SqDbTest debug: #testAddMemberToGroup"
	| db board1 member1 |
	db := SqSquareDB default.
	board1 := SqBoard id: SqUid new name: 'NewBoard1'.
	db registerGroup: board1.
	member1 := SqUser id: 'NewUser1' password: 'auau'.
	db addMember: member1 to: board1.
	self
		should: [(db membersOf: board1)
				includes: member1].
	db removeGroup: board1! !

!SqDbTest methodsFor: 'testing' stamp: 'mu 5/9/2006 14:42'!
testAddRemoveGroup
	"SqDbTest debug: #testAddRemoveGroup"
	| originalSize db group1 group2 |
	
	db _ SqSquareDB default.
	group1 _ SqBoard id: SqUid new name: 'NewBoard1'.
	group2 _ SqBoard id: SqUid new name: 'NewBoard2'.

	originalSize _ db groups size.

	db registerGroup: group1.
	db registerGroup: group2.

	self should: [(db groups collect: [:each | each name]) asArray includesAllOf: #('NewBoard1' 'NewBoard2')].

	db removeGroup: group1.
	db removeGroup: group2.

	self should: [originalSize = db groups size]
	! !

!SqDbTest methodsFor: 'testing' stamp: 'mu 5/9/2006 14:43'!
testIncludesGroup
	"SqDbTest debug: #testIncludesGroup"
	| db group1 group2 group3 |
	
	db _ SqSquareDB default.
	group1 _ SqBoard id: SqUid new name: 'NewBoard1'.
	group2 _ SqBoard id: SqUid new name: 'NewBoard2'.
	group3 _ SqBoard id: SqUid new name: 'NewBoard3'.
	db registerGroup: group1.
	db registerGroup: group2.

	self should: [db includesGroup: group1].
	self should: [db includesGroup: group2].
	self shouldnt: [db includesGroup: group3].

	db removeGroup: group1.
	db removeGroup: group2.

	self shouldnt: [db includesGroup: group1].
	self shouldnt: [db includesGroup: group2].
	self shouldnt: [db includesGroup: group3].

	! !

!SqDbTest methodsFor: 'testing' stamp: 'mu 5/9/2006 14:43'!
testRemoveMemberFromGroup
	"SqDbTest debug: #testRemoveMemberFromGroup"
	| db board1 member1 member2 |
	db := SqSquareDB default.
	board1 := SqBoard id: SqUid new name: 'NewBoard1'.
	db registerGroup: board1.
	member1 := SqUser id: 'NewUser1' password: 'auau'.
	member2 := SqUser id: 'NewUser2' password: 'auau'.

	db addMembers: {member1. member2} to: board1.

	self
		should: [(db membersOf: board1)
				includes: member1].	
	self
		should: [(db membersOf: board1)
				includes: member2].

	db removeMember: member1 from: board1.

	self
		shouldnt: [(db membersOf: board1)
				includes: member1].

	db removeGroup: board1! !

!SqDbTest methodsFor: 'testing' stamp: 'mu 5/9/2006 14:43'!
testRemoveMembersFromGroup
	"SqDbTest debug: #testRemoveMembersFromGroup"
	| db board1 member1 member2 member3 |
	db := SqSquareDB default.
	board1 := SqBoard id: SqUid new name: 'NewBoard1'.
	db registerGroup: board1.
	member1 := SqUser id: 'NewUser1' password: 'auau'.
	member2 := SqUser id: 'NewUser2' password: 'auau'.
	member3 := SqUser id: 'NewUser3' password: 'auau'.

	db addMembers: {member1. member2. member3} to: board1.

	self
		should: [(db membersOf: board1)
				includes: member1].	
	self
		should: [(db membersOf: board1)
				includes: member2].
	self
		should: [(db membersOf: board1)
				includes: member3].	

	db removeMembers: {member1. member3} from: board1.

	self
		shouldnt: [(db membersOf: board1)
				includes: member1].
	self
		shouldnt: [(db membersOf: board1)
				includes: member3].
	self
		should: [(db membersOf: board1)
				includes: member2].

	db removeGroup: board1! !

ElServer subclass: #SqAudioChatServerFacade
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqSquare-Server-AudioChat'!

SqAudioChatServerFacade class
	instanceVariableNames: 'default'!

!SqAudioChatServerFacade class methodsFor: 'instance creation' stamp: 'mu 3/27/2006 11:21'!
default
	default
		ifNil: [default := self new].
	^ default! !

!SqAudioChatServerFacade class methodsFor: 'instance creation' stamp: 'mu 3/27/2006 11:21'!
initialize
	default _ nil! !

!SqAudioChatServerFacade methodsFor: 'defaults' stamp: 'minami 5/11/2006 14:54'!
audioChatServerClass
	^ SqAudioChatServer! !

!SqAudioChatServerFacade methodsFor: 'defaults' stamp: 'minami 5/11/2006 14:52'!
commandServerClass
	^ SqAudioChatCommandServer! !

!SqAudioChatServerFacade methodsFor: 'testing' stamp: 'mu 3/27/2006 11:23'!
isRunning
	isRunning ifNil: [^false].
	^isRunning! !

!SqAudioChatServerFacade methodsFor: 'actions' stamp: 'minami 5/19/2006 14:49'!
restartInvalidConnectionHandlers
	self isRunning ifFalse: [^ self].

	commandServer restartInvalidConnectionHandler.
	chatServer restartInvalidConnectionHandler.
	! !

SBDomainViewer subclass: #SqBoardViewer
	instanceVariableNames: 'commentPanel'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqSquare-Server-WebUI'!

!SqBoardViewer methodsFor: 'accessing' stamp: 'mu 4/18/2006 12:29'!
board
	^model! !

!SqBoardViewer methodsFor: 'builder' stamp: 'mu 4/18/2006 12:28'!
buildActionPanel
	^actionPanel := SqBoardActionPanel parent: self! !

!SqBoardViewer methodsFor: 'builder' stamp: 'mu 6/5/2006 17:18'!
buildViewerWith: html 
	super buildViewerWith: html.
	viewer
		ifNotNilDo: [:v | v
				imageActionBlock: [:imageFile | html
						anchorWithAction: [self launch]
						do: [html
								image: (html
										urlForDocument: (SBComponent formFromFile: imageFile))]]].
	^ viewer! !

!SqBoardViewer methodsFor: 'actions' stamp: 'mu 5/26/2006 15:47'!
commitDelete: aModel
	self db removeBoard: aModel.
	
	SqServer default removeBoardServerAt: aModel boardId ! !

!SqBoardViewer methodsFor: 'actions' stamp: 'mu 4/14/2006 16:41'!
commitEdit: aModel 
	self db registerBoard: aModel! !

!SqBoardViewer methodsFor: 'actions' stamp: 'mu 4/18/2006 16:36'!
launch
	
	self call: ((SqBoardPluginLauncher parent: self) board: self board)! !

!SqBoardViewer methodsFor: 'builder' stamp: 'mu 5/26/2006 16:26'!
metaobjectOf: aModel
	^aModel metaobjectWithThumbnail.! !

!SqBoardViewer methodsFor: 'rendering' stamp: 'mu 5/26/2006 16:46'!
renderContentOn: html 
	html render: self actionPanel.
	html render: (self viewerWith: html).! !

!SqDomainModel methodsFor: '*SqSquare-Server' stamp: 'mu 4/17/2006 17:03'!
addAttributesTo: metaobject

	self addDefaultAttributesTo: metaobject. 
	
	^metaobject! !

!SqDomainModel methodsFor: '*SqSquare-Server' stamp: 'mu 6/8/2006 14:06'!
addDefaultAttributesTo: metaobject

	metaobject textAttribute: #id do: [ :attribute | 
		attribute
			label: 'id' translated;
			maxLength: 15;
			addRequiredRule ].

	metaobject textAttribute: #name do: [ :attribute | 
		attribute
			label: 'name' translated;
			maxLength: 20;
			addRequiredRule ].

	metaobject textAttribute: #updatedTime do: [ :attribute | 
		attribute
			label: 'last update' translated;
			readOnly ].

	
	^metaobject! !

!SqDomainModel methodsFor: '*SqSquare-Server' stamp: 'mu 4/14/2006 15:40'!
db
	^SqSquareDB default! !

!SqDomainModel methodsFor: '*SqSquare-Server' stamp: 'mu 4/13/2006 14:34'!
metaobject
	| metaobject |
	metaobject := SBMetaobject for: self.
	self addAttributesTo: metaobject.
	^metaobject! !

!SqDomainModel methodsFor: '*SqSquare-Server' stamp: 'mu 5/24/2006 13:57'!
metaobjectWithThumbnail
	| metaobject |
	metaobject := SBMetaobject for: self.
	metaobject
		fileUploadAttribute: #thumbnail
		do: [:attribute | attribute label: 'thumbnail' translated;
				 withFileName: false;
				 fileGetSelector: #thumbnail;
				 addRequiredRule].
	self addAttributesTo: metaobject.
	^ metaobject! !

!SqDomainModel methodsFor: '*SqSquare-Server' stamp: 'mu 4/14/2006 15:40'!
removeFromDb! !

!SqDomainModel methodsFor: '*SqSquare-Server' stamp: 'mu 4/14/2006 15:37'!
updateToDb! !

SBMenuComponent subclass: #SqMenuComponent
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqSquare-Server-WebUI'!

!SqMenuComponent methodsFor: 'factory' stamp: 'mu 4/14/2006 11:49'!
adminItemComponentNames
	^#(#SqUserManagement #SqBoardManagement)! !

!SqMenuComponent methodsFor: 'factory' stamp: 'mu 4/12/2006 14:14'!
authenticator
	^self db! !

!SqMenuComponent methodsFor: 'factory' stamp: 'mu 4/12/2006 14:14'!
db
	^SqSquareDB default! !

!SqMenuComponent methodsFor: 'factory' stamp: 'mu 4/13/2006 12:23'!
introductionClass

	^SqIntroduction! !

!SqMenuComponent methodsFor: 'factory' stamp: 'mu 6/8/2006 12:18'!
itemComponentNames
	^#(#SqIntroduction #SqBoardsList #SqUserRegistration)! !

!SqMenuComponent methodsFor: 'factory' stamp: 'mu 4/18/2006 11:21'!
userItemComponentNames
	^#()! !

SBActionPanel subclass: #SqBoardActionPanel
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqSquare-Server-WebUI'!

!SqBoardActionPanel methodsFor: 'rendering' stamp: 'mu 6/19/2006 14:09'!
renderContentOn: html 
	html
		table: [html
				tableRow: [html
						tableData: [html
								anchorWithAction: [self parent back]
								text: 'back' translated].
					self renderLaunchOn: html.
					self withEdit
						ifTrue: [self renderEditOn: html].
					self isAdminAuthenticated
						ifTrue: [self renderRollbackOn: html.
							self renderDeleteOn: html]]]! !

!SqBoardActionPanel methodsFor: 'rendering' stamp: 'mu 6/8/2006 14:00'!
renderLaunchOn: html 
	html
		tableData: [html
				anchorWithAction: [self parent launch]
				text: 'launch' translated]! !

!SqBoardActionPanel methodsFor: 'rendering' stamp: 'mu 6/19/2006 14:19'!
renderRollbackOn: html 
	html
		tableData: [html
				anchorWithAction: [self
						call: ((SqBoardRollbackList parent: self)
								board: self parent board)]
				text: 'rollback' translated0]! !

!FileDirectory methodsFor: '*SqSquare-Server-enumeration' stamp: 'mu 6/19/2006 13:53'!
deleteFilesMatches: selectionBlock 
	| deleted |
	deleted _ self filesMatches: selectionBlock. 
	deleted
		do: [:each | self
				deleteFileNamed: each name
				ifAbsent: []]! !

!FileDirectory methodsFor: '*SqSquare-Server-enumeration' stamp: 'MU 2/7/2006 18:10'!
directoriesMatches: selectionBlock
	^self directoryEntries select: selectionBlock! !

!FileDirectory methodsFor: '*SqSquare-Server-enumeration' stamp: 'MU 2/7/2006 18:08'!
directoryEntries
	^self entries select: [:e | e isDirectory]! !

!FileDirectory methodsFor: '*SqSquare-Server-enumeration' stamp: 'MU 2/7/2006 18:08'!
fileEntries
	^self entries reject: [:e | e isDirectory]! !

!FileDirectory methodsFor: '*SqSquare-Server-enumeration' stamp: 'MU 2/7/2006 18:10'!
filesMatches: selectionBlock
	^self fileEntries select: selectionBlock! !

!FileDirectory methodsFor: '*SqSquare-Server-enumeration' stamp: 'MU 2/7/2006 18:24'!
latestFileMatches: selectionBlock
	| entries |
	entries _ self filesMatches: selectionBlock.
	entries ifEmpty: [^nil].
	^(entries sort: [:a :b | a modificationTime > b modificationTime]) first! !

!FileDirectory methodsFor: '*SqSquare-Server-enumeration' stamp: 'mu 4/7/2006 14:59'!
oldestFileMatches: selectionBlock
	| entries |
	entries _ self filesMatches: selectionBlock.
	entries ifEmpty: [^nil].
	^(entries sort: [:a :b | a modificationTime > b modificationTime]) last! !

SqWarning subclass: #SqEventRenumbered
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqSquare-Server'!

SBDomainList subclass: #SqBoardsList
	instanceVariableNames: 'jumpComboBox sortOption'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqSquare-Server-WebUI'!

!SqBoardsList class methodsFor: 'constants' stamp: 'mu 4/17/2006 17:08'!
maxItemsPerPage
	^10! !

!SqBoardsList methodsFor: 'accessing' stamp: 'mu 4/17/2006 17:27'!
boards
	^self items! !

!SqBoardsList methodsFor: 'accessing' stamp: 'mu 4/17/2006 17:27'!
boards: boards
	self items: boards! !

!SqBoardsList methodsFor: 'builder' stamp: 'mu 6/8/2006 11:56'!
buildJumpComboBox
	| comboBox selectList |
	comboBox := SBComboBox parent: self.
	selectList := {#decTime -> 'time order' translated. #incName -> 'name order' translated}.
	comboBox listWithBlank: selectList;
		 label: 'Jump';
		 labelsBlock: [:each | each value];
		 id: #sortOptionChanged.
	^ comboBox! !

!SqBoardsList methodsFor: 'accessing' stamp: 'mu 6/8/2006 11:31'!
children
	^{batchList. jumpComboBox}! !

!SqBoardsList methodsFor: 'callback' stamp: 'mu 6/14/2006 13:40'!
comboBoxSelected: item from: aspect 
	(item isString and: [item isEmpty]) ifTrue: [^nil].
	aspect == #sortOptionChanged
		ifTrue: [
			items _ nil.
			self sortOption: item key]! !

!SqBoardsList methodsFor: 'accessing' stamp: 'mu 4/18/2006 10:59'!
db
	^SqSquareDB default! !

!SqBoardsList methodsFor: 'private' stamp: 'mu 4/18/2006 11:53'!
hiddenAttributeNames
	"override"
	^#(id)! !

!SqBoardsList methodsFor: 'callback' stamp: 'mu 6/8/2006 12:00'!
initItemsFrom: start to: end
	^items := self db boardsFrom: start to: end sortOption: self sortOption! !

!SqBoardsList methodsFor: 'initialize-release' stamp: 'mu 6/8/2006 11:40'!
initialize
	super initialize.
	jumpComboBox _ nil.
	sortOption _ #decTime! !

!SqBoardsList methodsFor: 'accessing' stamp: 'mu 6/8/2006 11:22'!
jumpComboBox
	^ jumpComboBox
		ifNil: [jumpComboBox := self buildJumpComboBox]! !

!SqBoardsList methodsFor: 'accessing' stamp: 'mu 6/8/2006 11:22'!
jumpComboBox: aSBComboBox
	jumpComboBox _ aSBComboBox! !

!SqBoardsList methodsFor: 'private' stamp: 'mu 4/18/2006 11:32'!
maxItemsSize
	"override"
	^self db sizeOfBoards! !

!SqBoardsList methodsFor: 'private' stamp: 'mu 5/24/2006 14:36'!
metaobjectOf: board
	
	^board metaobjectWithThumbnail	! !

!SqBoardsList methodsFor: 'accessing' stamp: 'mu 6/8/2006 14:04'!
name

	^ 'board list' translated! !

!SqBoardsList methodsFor: 'rendering' stamp: 'mu 6/8/2006 11:14'!
renderContentOn: html 
	self renderHeaderOn: html. 
	self renderDescriptionOn: html.
	self renderJumperOn: html. 
	self renderTableOn: html! !

!SqBoardsList methodsFor: 'rendering' stamp: 'mu 6/8/2006 14:04'!
renderDescriptionOn: html 
	| msg |
	super renderDescriptionOn: html.
	msg := '{1} items.' translated.
	html
		text: (msg format: {self class defaultNumberOfItems min: self boards size})! !

!SqBoardsList methodsFor: 'rendering' stamp: 'mu 6/8/2006 11:28'!
renderJumperOn: html 
	html render: self jumpComboBox! !

!SqBoardsList methodsFor: 'accessing' stamp: 'mu 6/8/2006 11:40'!
sortOption
	^sortOption! !

!SqBoardsList methodsFor: 'accessing' stamp: 'mu 6/8/2006 11:40'!
sortOption: aSymbol
	sortOption _ aSymbol! !

!SqBoardsList methodsFor: 'private' stamp: 'mu 6/14/2006 23:37'!
viewerFor: item on: html 
	| viewer |
	viewer _ super viewerFor: item on: html.
	viewer
		anchorMatchBlock: [:nm | nm = 'name' translated];
		
		anchorActionBlock: [:label | html
				anchorWithAction: [self
						call: (SqBoardViewer parent: self model: item db: self db)]
				text: item name];
		
		imageActionBlock: [:imageFile | html
				anchorWithAction: [self
						call: ((SqBoardPluginLauncher parent: self) board: item)]
				do: [html
						image: (html
								urlForDocument: (SBComponent formFromFile: imageFile))]].
	^ viewer! !

SBBodyComponent subclass: #SqBodyComponent
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqSquare-Server-WebUI'!

!SqBodyComponent methodsFor: 'factory' stamp: 'mu 4/13/2006 12:23'!
defaultPageClass
	^SqIntroduction! !

!SqBodyComponent methodsFor: 'factory' stamp: 'mu 4/12/2006 15:39'!
defaultPageTitle
	^'Welcome'! !

!SqBodyComponent methodsFor: 'factory' stamp: 'mu 5/26/2006 11:33'!
defaultPanelClass
	^SBSideBarPanel! !

!SqBodyComponent methodsFor: 'factory' stamp: 'mu 6/8/2006 14:17'!
mainTitle
	^'Welcome' translated! !

!SqBodyComponent methodsFor: 'factory' stamp: 'mu 4/13/2006 12:23'!
menuClass
	^SqMenuComponent! !

SBObjectDB subclass: #SqSquareDB
	instanceVariableNames: 'users'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqSquare-Server-Storage'!

!SqSquareDB class methodsFor: 'class initialization' stamp: 'MU 2/27/2006 14:12'!
createDb
	"SqSquareDB createDb"
	self default dbAccessorClass initialize.
	self registerDefaults! !

!SqSquareDB class methodsFor: 'utilities' stamp: 'minami 2/22/2006 16:46'!
defaultUsers

	^ {
		{'aaa'. 'bbb'}. 
		{'bbb'. 'ccc'}. 
		{'ccc'. 'ddd'}.

		{'user01'. 'user01'}.
		{'user02'. 'user02'}.
		{'user03'. 'user03'}.
		{'user04'. 'user04'}.
		{'user05'. 'user05'}.
		{'user06'. 'user06'}.
		{'user07'. 'user07'}.
		{'user08'. 'user08'}.
		{'user09'. 'user09'}.
		{'user10'. 'user10'}.
	}! !

!SqSquareDB class methodsFor: 'utilities' stamp: 'MU 2/21/2006 16:05'!
registerDefaults
	"SqSquareDB registerDefaults"
	self registerDefaultUsers! !

!SqSquareDB class methodsFor: 'utilities' stamp: 'mu 6/5/2006 22:44'!
registerDefaultUsers

	self default basicUsers isEmpty ifFalse: [^self].
	 
	self defaultUsers
		do: [:each | self default
				registerUser: (SqUser id: each first password: each second)].
	self default
		registerUser: ((SqUser id: 'root' password: 'arakeita')
				beAdmin)! !

!SqSquareDB methodsFor: 'event management' stamp: 'mu 4/7/2006 14:35'!
addEvents: events to: boardId 
	| nextWantedNumber eventsStream found stored |
	events isEmptyOrNil ifTrue: [^self].
	nextWantedNumber := (self latestEventNumberOf: boardId) + 1.
	
	eventsStream _ events readStream.
	found _ false.
	[eventsStream atEnd or: [found]] whileFalse: [|next|
		next _ eventsStream next.
		next eventId = nextWantedNumber ifTrue: [found _ true].
	].


	found ifFalse: [self basicAddEvents: (SqUtil renumber: events from: nextWantedNumber -1) to: boardId.
		^SqEventRenumbered signal: #renumbered. 
	 ].

	stored _ eventsStream skip: -1; upToEnd.
	self basicAddEvents: stored to: boardId! !

!SqSquareDB methodsFor: 'board management' stamp: 'mu 4/14/2006 15:13'!
addMembers: users to: aBoard
	| result |
	result := true.
	self
		commitDo: [:tra | 
			| dict board | 
			dict := self containerNamed: self boardsKey on: tra.
			board := dict
						at: aBoard id
						ifAbsent: [result := false].
			board
				ifNotNilDo: [:b | 
					users do: [:each | b addMember: each].
					tra markDirty: b]].
	^ result! !

!SqSquareDB methodsFor: 'board management' stamp: 'mu 4/14/2006 15:03'!
addMember: anUser to: aBoard 
	| result |
	result := true.
	self
		commitDo: [:tra | 
			| dict board | 
			dict := self containerNamed: self boardsKey on: tra.
			board := dict
						at: aBoard id
						ifAbsent: [result := false].
			board
				ifNotNilDo: [:b | 
					b addMember: anUser.
					tra markDirty: b]].
	^ result! !

!SqSquareDB methodsFor: 'event management' stamp: 'mu 6/5/2006 21:09'!
allEventsOf: boardId 
	| ret |
	ret := self containerNamed: self eventsKey at: boardId.
	^ ret
		ifNil: [#()]
		ifNotNil: [ret collect: [:e | SqEvent fromStoredBytes: e]]! !

!SqSquareDB methodsFor: 'event management' stamp: 'mu 6/5/2006 20:52'!
basicAddEvents: events to: boardId 
	events
		ifEmpty: [^ self].
	^ self
		containerNamed: self eventsKey
		at: boardId
		addAll: (events collect: [:e | e asStoredBytes])
		ifAbsentCreateClass: OrderedCollection! !

!SqSquareDB methodsFor: 'event management' stamp: 'mu 6/5/2006 20:52'!
basicEventsFrom: dict of: boardId 
	| rowEvents |
	rowEvents _ dict
				at: boardId
				ifAbsent: [^ #()].
	^ rowEvents
		collect: [:e | SqEvent fromStoredBytes: e]! !

!SqSquareDB methodsFor: 'user management' stamp: 'mu 6/5/2006 22:44'!
basicUsers
	^super users! !

!SqSquareDB methodsFor: 'board management' stamp: 'mu 4/14/2006 14:39'!
boards
	^self groups! !

!SqSquareDB methodsFor: 'board management' stamp: 'mu 4/14/2006 14:39'!
boardsFrom: startIndex to: endIndex sortOption: option 
	^self groupsFrom: startIndex to: endIndex sortOption: option ! !

!SqSquareDB methodsFor: 'constants' stamp: 'mu 4/14/2006 14:58'!
boardsKey
	^self groupsKey! !

!SqSquareDB methodsFor: 'event management' stamp: 'mu 6/19/2006 13:35'!
discardEventsFrom: start of: boardId 
	| events |
	events _ #().
	self
		commitDo: [:tra | 
			| dict allEvents done | 
			dict _ self containerNamed: self eventsKey on: tra.
			allEvents _ self basicEventsFrom: dict of: boardId.
			done _ false.
			allEvents
				withIndexDo: [:each :index | (done not
							and: [each eventId = start])
						ifTrue: [| col | 
							col _ dict at: boardId.
							dict at: boardId
								put: (col copyFrom: 1 to: index).
							dict markDirty.
							done _ true]]].
	^ events! !

!SqSquareDB methodsFor: 'event management' stamp: 'mu 6/5/2006 19:23'!
eventsFrom: start to: end of: boardId 
	| events |
	events := #().
	self
		readDo: [:tra | 
			| dict allEvents | 
			dict := self containerNamed: self eventsKey on: tra.
			allEvents := self basicEventsFrom: dict of: boardId. 
			events := self
						safeEventsCopyFrom: start
						to: (end min: allEvents size)
						using: allEvents].
	^ events! !

!SqSquareDB methodsFor: 'constants' stamp: 'MU 2/21/2006 16:49'!
eventsKey
	^'events'! !

!SqSquareDB methodsFor: 'group management' stamp: 'MU 2/21/2006 16:36'!
groups
	^self allContainerContentsNamed: self groupsKey! !

!SqSquareDB methodsFor: 'group management' stamp: 'mu 4/14/2006 10:59'!
groupsFrom: startIndex to: endIndex sortOption: option 
	| sortBlock groups |
	sortBlock := self selectComparerFrom: option.
	groups := OrderedCollection new.
	self
		readDo: [:tra | 
			| dict sorted start end | 
			dict := self containerNamed: self groupsKey on: tra.
			sorted := dict values asSortedCollection: sortBlock.
			sorted
				ifEmpty: [^ #()].
			start := startIndex min: sorted size max: 1.
			end := endIndex min: sorted size max: start.
			start
				to: end
				do: [:idx | groups add: (sorted at: idx) yourself]].
	^ groups! !

!SqSquareDB methodsFor: 'constants' stamp: 'MU 2/21/2006 16:49'!
groupsKey
	^'groups'! !

!SqSquareDB methodsFor: 'board management' stamp: 'mu 4/14/2006 14:39'!
includesBoard: aBoard
	^self includesGroup: aBoard! !

!SqSquareDB methodsFor: 'group management' stamp: 'mu 4/14/2006 11:03'!
includesGroup: aGroup
	^(self containerNamed: self groupsKey at: aGroup id) notNil! !

!SqSquareDB methodsFor: 'initialize' stamp: 'MU 2/21/2006 16:49'!
initContainers
	"SqSquareDB default initContainers"
	self containerNamed: self eventsKey ifNoContainerCreateBTreeDict: self keySize ! !

!SqSquareDB methodsFor: 'initialize' stamp: 'MU 2/21/2006 16:48'!
initialize
	super initialize.
	self initContainers! !

!SqSquareDB methodsFor: 'serial numbers' stamp: 'mu 5/11/2006 14:23'!
initSerialNumberFor: domainKey 
	| ret |
	ret _ nil.
	self
		commitDo: [:tra | 
			| dict | 
			dict _ self dictionaryNamed: self serialNumbersKey on: tra.
			ret _ (dict
						at: domainKey
						ifAbsentPut: [0]) yourself.
			ret _ 0.
			dict at: domainKey put: ret].
	^ ret! !

!SqSquareDB methodsFor: 'user management' stamp: 'mu 4/14/2006 16:36'!
initUsers
	users _ nil! !

!SqSquareDB methodsFor: 'event management' stamp: 'mu 6/5/2006 22:05'!
latestEventNumberOf: boardId
	| number |
	number _ 0.
	self
		readDo: [:tra | 
			| dict allEvents | 
			dict := self containerNamed: self eventsKey on: tra.
			allEvents := self basicEventsFrom: dict of: boardId.
			number _ allEvents ifEmpty: [0] ifNotEmpty: [allEvents last eventId]].
	^ number! !

!SqSquareDB methodsFor: 'event management' stamp: 'mu 6/5/2006 19:24'!
latestEventsFrom: start of: boardId 
	| events |
	events := #().
	self
		readDo: [:tra | 
			| dict allEvents | 
			dict := self containerNamed: self eventsKey on: tra.
			allEvents := self basicEventsFrom: dict of: boardId.
			events := self
						safeEventsCopyFrom: start
						to: allEvents size
						using: allEvents].
	^ events! !

!SqSquareDB methodsFor: 'board management' stamp: 'mu 4/14/2006 16:47'!
membersOf: aBoard
	| result |
	result := #().
	self
		readDo: [:tra | 
			| dict board | 
			dict := self containerNamed: self boardsKey on: tra.
			board := dict
						at: aBoard id
						ifAbsent: [].
			board
				ifNotNilDo: [:b | 
					result _ b members]].
	^ result asSortedCollection! !

!SqSquareDB methodsFor: 'serial numbers' stamp: 'mu 5/11/2006 14:20'!
nextSerialNumberFor: domainKey 
	| ret |
	ret _ nil.
	self
		commitDo: [:tra | 
			| dict | 
			dict _ self dictionaryNamed: self serialNumbersKey on: tra.
			ret _ (dict
						at: domainKey
						ifAbsentPut: [0]) yourself.
			ret _ ret + 1.
			dict at: domainKey put: ret].
	^ ret! !

!SqSquareDB methodsFor: 'private' stamp: 'MU 2/23/2006 15:15'!
outOfRangeFrom: start to: end for: events 
	| outOfRange |
	outOfRange := SqOutOfRange
				from: start
				to: end
				actualFrom: (events
						ifEmpty: [-1]
						ifNotEmpty: [events first eventId])
				to: events size.
	^ outOfRange! !

!SqSquareDB methodsFor: 'board management' stamp: 'mu 4/14/2006 14:40'!
registerBoard: aBoard
	^self registerGroup: aBoard! !

!SqSquareDB methodsFor: 'group management' stamp: 'mu 5/22/2006 13:40'!
registerGroup: aGroup
	(self includesGroup: aGroup) ifFalse: [aGroup serialNumber: (self nextSerialNumberFor: self groupsKey)].
	self
		containerNamed: self groupsKey
		at: aGroup id
		put: aGroup! !

!SqSquareDB methodsFor: 'user management' stamp: 'mu 6/5/2006 22:48'!
registerUser: anUser 
	self initUsers.
	((self includesUser: anUser) not
			or: [anUser serialNumber = 0])
		ifTrue: [anUser
				serialNumber: (self nextSerialNumberFor: self usersKey)].
	super registerUser: anUser! !

!SqSquareDB methodsFor: 'board management' stamp: 'mu 4/14/2006 14:40'!
removeAllBoards
	self removeAllGroups! !

!SqSquareDB methodsFor: 'event management' stamp: 'MU 2/23/2006 11:49'!
removeAllEventsFrom: boardId 
	self
		containerNamed: self eventsKey
		removeKey: boardId
		ifAbsent: []! !

!SqSquareDB methodsFor: 'group management' stamp: 'MU 2/22/2006 14:21'!
removeAllGroups
	^self removeContainerNamed: self groupsKey! !

!SqSquareDB methodsFor: 'board management' stamp: 'mu 4/14/2006 14:41'!
removeBoard: aBoard
	self removeGroup: aBoard! !

!SqSquareDB methodsFor: 'event management' stamp: 'mu 6/5/2006 19:25'!
removeEvents: events from: boardId 
	self
		containerNamed: self eventsKey
		at: boardId
		removeAll: (events collect: [:e | e asCompactBytes])
		! !

!SqSquareDB methodsFor: 'group management' stamp: 'MU 2/21/2006 16:35'!
removeGroup: aGroup
	^self
		containerNamed: self groupsKey
		removeKey: aGroup id
		! !

!SqSquareDB methodsFor: 'board management' stamp: 'mu 4/14/2006 15:21'!
removeMembers: users from: aBoard
	| result |
	result := true.
	self
		commitDo: [:tra | 
			| dict board | 
			dict := self containerNamed: self boardsKey on: tra.
			board := dict
						at: aBoard id
						ifAbsent: [result := false].
			board
				ifNotNilDo: [:b | 
					users do: [:each | b removeMember: each ifAbsent: [result := false]].
					tra markDirty: b]].
	^ result! !

!SqSquareDB methodsFor: 'board management' stamp: 'mu 4/14/2006 15:20'!
removeMember: anUser from: aBoard
	| result |
	result := true.
	self
		commitDo: [:tra | 
			| dict board | 
			dict := self containerNamed: self boardsKey on: tra.
			board := dict
						at: aBoard id
						ifAbsent: [result := false].
			board
				ifNotNilDo: [:b | 
					b removeMember: anUser ifAbsent: [result := false].
					tra markDirty: b]].
	^ result! !

!SqSquareDB methodsFor: 'user management' stamp: 'mu 4/14/2006 16:37'!
removeUser: anUser  
	self initUsers.
	super removeUser: anUser  ! !

!SqSquareDB methodsFor: 'private' stamp: 'MU 2/23/2006 15:15'!
safeEventsCopyFrom: start to: end using: allEvents 
	| events |
	[events := allEvents copyFrom: start to: end]
		on: Error
		do: [:ex | ex
				resignalAs: (self
						outOfRangeFrom: start
						to: end
						for: allEvents)].
	^ events! !

!SqSquareDB methodsFor: 'constants' stamp: 'mu 5/11/2006 14:20'!
serialNumbersKey
	^'serialNumbers'! !

!SqSquareDB methodsFor: 'board management' stamp: 'mu 6/14/2006 09:09'!
setBoardUpdatedTimeAt: boardId 
	| board |
	self
		commitDo: [:tra | 
			| dict | 
			dict _ self containerNamed: self boardsKey on: tra.
			board _ dict
						at: boardId
						ifAbsent: [].
			board
				ifNotNilDo: [:b | 
					b computeUpdatedTime.
					tra markDirty: b]]! !

!SqSquareDB methodsFor: 'board management' stamp: 'mu 4/18/2006 11:32'!
sizeOfBoards
	^self sizeOfGroups! !

!SqSquareDB methodsFor: 'group management' stamp: 'mu 4/18/2006 11:32'!
sizeOfGroups
	^self sizeOfContainerNamed: self groupsKey! !

!SqSquareDB methodsFor: 'user management' stamp: 'mu 5/22/2006 16:09'!
userProfileAt: serial
	^self users detect: [:each | each serialNumber = serial] ifNone: []
! !

!SqSquareDB methodsFor: 'user management' stamp: 'mu 4/26/2006 12:24'!
userProfileOf: uid
	| user |
	user _ self userOf: uid.
	^user ifNotNilDo: [:u | u asProfile]! !

!SqSquareDB methodsFor: 'user management' stamp: 'mu 4/14/2006 16:44'!
users
	users ifNil: [users _ super users].
	^users asSortedCollection! !

!SqSquareDB methodsFor: 'constants' stamp: 'mu 5/11/2006 14:31'!
usersKey
	^'sqUsers'! !

WASystemConfiguration subclass: #SqSquareConfiguration
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqSquare-Server-WebUI'!

!SqSquareConfiguration methodsFor: 'attributes' stamp: 'mu 6/8/2006 14:43'!
attributes
	^{
		(WAStringAttribute key: #moreInfoUrl group: #SqSquare).
	}
		! !

!SqSquareConfiguration methodsFor: 'defaults' stamp: 'mu 6/12/2006 20:32'!
moreInfoUrl
	^'http://sqsq.jp/SqSquare'! !

RmtRpcAcceptor subclass: #SqRpcDispatcher
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqSquare-Server'!

!SqRpcDispatcher class methodsFor: 'class initialization' stamp: 'MU 1/20/2006 16:39'!
initialize
	"SqRpcDispatcher initialize"
	super initialize.
	receiverDictionary _ Dictionary new.
	self registerAllServices.

	self start! !

!SqRpcDispatcher class methodsFor: 'registering' stamp: 'minami 2/23/2006 11:41'!
registerAllServices
	"SqRpcDispatcher registerAllServices"
	self receiverDictionary at: #loginService put: SqLoginManager new.
	self receiverDictionary at: #userService put: SqUserManager new.
	self receiverDictionary at: #boardService put: SqBoardManager new.! !

!SqRpcDispatcher class methodsFor: 'actions' stamp: 'MU 1/20/2006 16:17'!
start
	"SqRpcDispatcher start"

	service _ (RmtTcpService on: SqSettings rpcListenPort) acceptorClass: self.
	service start! !

!SqRpcDispatcher class methodsFor: 'actions' stamp: 'MU 2/23/2006 18:18'!
stop
	"SqRpcDispatcher stop"
	service
		ifNotNilDo: [:ser | ser stop]! !

SBDomainManagement subclass: #SqBoardManagement
	instanceVariableNames: 'newBoard'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqSquare-Server-WebUI-Admin'!

!SqBoardManagement methodsFor: 'actions' stamp: 'mu 6/8/2006 14:01'!
accept
	self isAdminAuthenticated
		ifFalse: [self inform: 'Invalid operation - you are not logged in.' translated.
			^ self cancel].
	(self db includesUser: newBoard)
		ifTrue: [self inform: 'Duplicated board ids.' translated.
			^ self cancel].
	self db registerBoard: newBoard.
	newBoard := nil! !

!SqBoardManagement methodsFor: 'actions' stamp: 'mu 4/14/2006 12:14'!
addNewItem
	newBoard := self call: (SBEditorVisitor model: self domainClass new).
	! !

!SqBoardManagement methodsFor: 'accessing' stamp: 'mu 4/14/2006 14:56'!
boards
	^ self db
		boardsFrom: 1
		to: self maxItemsPerPage
		sortOption: self sortOption! !

!SqBoardManagement methodsFor: 'actions' stamp: 'mu 4/14/2006 12:09'!
cancel
	newBoard _ nil! !

!SqBoardManagement methodsFor: 'factory' stamp: 'mu 6/8/2006 14:02'!
createTableViewer
	tableViewer := super createTableViewer.
	tableViewer
		anchorMatchBlock: [:nm | nm = 'id' translated].
	^ tableViewer! !

!SqBoardManagement methodsFor: 'accessing' stamp: 'mu 4/14/2006 12:27'!
db
	^SqSquareDB default! !

!SqBoardManagement methodsFor: 'factory' stamp: 'mu 4/14/2006 12:15'!
domainClass
	^SqBoard! !

!SqBoardManagement methodsFor: 'callback' stamp: 'mu 4/14/2006 14:56'!
initItemsFrom: start to: end 
	^ self db
		boardsFrom: start
		to: end
		sortOption: self sortOption! !

!SqBoardManagement methodsFor: 'accessing' stamp: 'mu 6/8/2006 14:02'!
itemName
	^'board' translated! !

!SqBoardManagement methodsFor: 'accessing' stamp: 'mu 6/8/2006 14:07'!
name
	^'board administration' translated! !

!SqBoardManagement methodsFor: 'rendering' stamp: 'mu 4/14/2006 12:05'!
renderContentOn: html 
	super renderContentOn: html.
	self renderItem: newBoard on: html 

	! !

!SqBoardManagement methodsFor: 'accessing' stamp: 'mu 4/14/2006 12:17'!
rows
	^self boards! !

!SqBoardManagement methodsFor: 'accessing' stamp: 'mu 4/14/2006 12:17'!
titleHeading
	^self name! !

!SqBoardManagement methodsFor: 'actions' stamp: 'mu 4/14/2006 12:20'!
viewItem: aBoard
	self
		call: (SqBoardViewer
				parent: self
				model: aBoard
				db: self db)! !

SBUserRegistration subclass: #SqUserRegistration
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqSquare-Server-WebUI'!

!SqUserRegistration methodsFor: 'accessing' stamp: 'mu 6/8/2006 12:17'!
db
	^SqSquareDB default! !

!SqUserRegistration methodsFor: 'constants' stamp: 'mu 6/8/2006 12:38'!
initialUserHiddenAttributes
	^#(#isAdmin #birthday #male)! !

!SqUserRegistration methodsFor: 'factory' stamp: 'mu 6/8/2006 12:26'!
userDomainClass
	^SqUser! !

!SqUserRegistration methodsFor: 'constants' stamp: 'mu 6/8/2006 12:38'!
userHiddenAttributes
	^#(#isAdmin #birthday #password #updatedTime #age #male)! !

SBRootComponent subclass: #SqRootComponent
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqSquare-Server-WebUI'!

!SqRootComponent class methodsFor: 'constants' stamp: 'mu 4/13/2006 12:24'!
applicationName
	^'SqSquare'! !

!SqRootComponent class methodsFor: 'constants' stamp: 'mu 4/12/2006 17:10'!
logoString1
	"self readImageToBase64: 'sqsq2.gif' in: (FileDirectory default)"
	"self setLogo: 1"
	^ 'R0lGODlhIAEtAPfYAAAAADMAAGYAAJkAAMwAAP8AAAAzADMzAGYzAJkzAMwzAP8zAABmADNm
AGZmAJlmAMxmAP9mAACZADOZAGaZAJmZAMyZAP+ZAADMADPMAGbMAJnMAMzMAP/MAAD/ADP/
AGb/AJn/AMz/AP//AAAAMzMAM2YAM5kAM8wAM/8AMwAzMzMzM2YzM5kzM8wzM/8zMwBmMzNm
M2ZmM5lmM8xmM/9mMwCZMzOZM2aZM5mZM8yZM/+ZMwDMMzPMM2bMM5nMM8zMM//MMwD/MzP/
M2b/M5n/M8z/M///MwAAZjMAZmYAZpkAZswAZv8AZgAzZjMzZmYzZpkzZswzZv8zZgBmZjNm
ZmZmZplmZsxmZv9mZgCZZjOZZmaZZpmZZsyZZv+ZZgDMZjPMZmbMZpnMZszMZv/MZgD/ZjP/
Zmb/Zpn/Zsz/Zv//ZgAAmTMAmWYAmZkAmcwAmf8AmQAzmTMzmWYzmZkzmcwzmf8zmQBmmTNm
mWZmmZlmmcxmmf9mmQCZmTOZmWaZmZmZmcyZmf+ZmQDMmTPMmWbMmZnMmczMmf/MmQD/mTP/
mWb/mZn/mcz/mf//mQAAzDMAzGYAzJkAzMwAzP8AzAAzzDMzzGYzzJkzzMwzzP8zzABmzDNm
zGZmzJlmzMxmzP9mzACZzDOZzGaZzJmZzMyZzP+ZzADMzDPMzGbMzJnMzMzMzP/MzAD/zDP/
zGb/zJn/zMz/zP//zAAA/zMA/2YA/5kA/8wA//8A/wAz/zMz/2Yz/5kz/8wz//8z/wBm/zNm
/2Zm/5lm/8xm//9m/wCZ/zOZ/2aZ/5mZ/8yZ//+Z/wDM/zPM/2bM/5nM/8zM///M/wD//zP/
/2b//5n//8z/////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////
/////////////////////////////////yH5BAUBANgALAAAAAAgAS0AAAj/ALEJHEiwoMGD
CBMqXMiwocOHECNKnEixosWLGDNq3Mixo8ePIEOKHEmypMmTKFOqXMmypcuXMFc+m0lzZsia
NWPq3MlzJM6fzzoCxdmzqNGjEIcS3ag0J9KnUJ82pclxqs2oWLPqXErwqkauA71qHUvWJFiQ
Z8uqXUvSqU+qbOPKRQv3rdi5ePNSdBvxZ9i7fJNyrSswcFfCBYEudOv3YGO9UhE3tApYMkPK
QQ/f1Zw5cdOETh9zFg25qGGFmDv/3Ywas2fWhSVTRphT8WilpY2Spn027W7HvQOfjs0acVps
tguPfg0798vkwC0jtwzd4PHplZufVs05unTi3FeH/3cOcyhv7dLNewdt/Ptw9puvg4c/nvxz
9d3P18e++71w99/pZ9172C0WoH0tQUcggeARtmB76DXXIH8DSsgggwgmOFiAGDbY2YPZ7eff
VBXux19qGR61oYUHLjcffczBWOJnMdKHYoq6hcZhiy6CyN2I8YHl246pSYijSzqyaGSJLwqY
33rMhWeYj0fGlaSJHeZH5ZNMulgjl11WqdaVrS1Z45biyZgmlGB+KSZZV4rIo5ZA/ghgiE62
6eabWK0op5nifXinnRESWt+UB2bJJ0rF8XUdot6p9iiEjVIaaZj6ibioSq7dFuWMuHnqZaC3
Selgfy2SNuemX80GaqiBwn9aKn6x0jgrVf+Z2SmrbZFoo6wnVhclsB7WRp16Qs5pFa8pCZtp
ZsM5Gym0g3pVp6mSrhosoMwipahF33Yrrkjh7qXtuOh6VO5E66br7kXt9nXuu/RWFK9g3Nar
L7vz8pvvvgA7dO9DAwds8Jp2mXjwwgw37PDDEEcs8cQUaxQQADs='
	
	! !

!SqRootComponent class methodsFor: 'registering' stamp: 'mu 6/8/2006 14:45'!
registerAsApplication
	| app |
	app := super registerAsApplication.
	app configuration addAncestor: SqSquareConfiguration localConfiguration.
	^app! !

!SqRootComponent class methodsFor: 'registering' stamp: 'mu 4/13/2006 12:22'!
styleClasses
	^{SqStyles}! !

!SqRootComponent methodsFor: 'factory' stamp: 'mu 4/13/2006 12:22'!
bodyClass
	^SqBodyComponent! !

!SqRootComponent methodsFor: 'factory' stamp: 'mu 4/12/2006 16:22'!
headerTitle
	"override"
	^'SqSquare'! !

SqAcceptor subclass: #SqBoardServerAcceptor
	instanceVariableNames: 'boardServer'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqSquare-Server'!

!SqBoardServerAcceptor methodsFor: 'accessing' stamp: 'minami 1/28/2006 09:09'!
boardId
	^self accessId boardId! !

!SqBoardServerAcceptor methodsFor: 'accessing' stamp: 'MU 1/31/2006 18:57'!
boardServer
	^boardServer! !

!SqBoardServerAcceptor methodsFor: 'receiving' stamp: 'mu 3/17/2006 12:39'!
handleReceived: messageObject 
	| server |
	server := self boardServer.
	server ifNil: [SqWarning warnLog: 'Board not found'. ^self].
	
	server addEvent: messageObject.
	server broadcast: messageObject! !

!SqBoardServerAcceptor methodsFor: 'receiving' stamp: 'mu 3/17/2006 12:38'!
process: message 
	super process: message.
	message type == #negotiate
		ifTrue: [
			| bId |
			boardServer _  SqServer default 
				boardServerAt: (bId := self accessId boardId) 
				ifAbsentPut: [(SqBoardServer boardId: bId) start; yourself]].

	
! !

!SqGroup methodsFor: '*SqSquare-Server' stamp: 'mu 4/17/2006 17:02'!
addAttributesTo: metaobject 
	super addAttributesTo: metaobject.
	self addMembersAttributesTo: metaobject ! !

!SqGroup methodsFor: '*SqSquare-Server' stamp: 'mu 6/8/2006 14:07'!
addMembersAttributesTo: metaobject 
	metaobject
		multipleRelationshipAttribute: #members
		do: [:attribute | attribute label: 'members' translated;
				
				relationshipTo: [SqSquareDB default users]
				formatWith: [:value | value asString , ' ']].

	^metaobject! !

!SqGroup methodsFor: '*SqSquare-Server' stamp: 'mu 4/14/2006 15:41'!
removeFromDb
	self db removeGroup: self! !

!SqGroup methodsFor: '*SqSquare-Server' stamp: 'mu 4/14/2006 15:48'!
updateToDb
	self db registerGroup: self! !

TdTimer subclass: #SqAudioChatMaintainer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqSquare-Server-AudioChat'!

!SqAudioChatMaintainer class methodsFor: 'constants' stamp: 'minami 5/19/2006 14:39'!
defaultIntervalSeconds
	^ 10! !

!SqAudioChatMaintainer class methodsFor: 'constants' stamp: 'minami 5/19/2006 14:39'!
defaultPriority
	^ Processor userBackgroundPriority! !

!SqAudioChatMaintainer methodsFor: 'actions' stamp: 'minami 5/19/2006 14:40'!
perform
	self restartInvalidConnectionHandlers! !

!SqAudioChatMaintainer methodsFor: 'actions' stamp: 'minami 5/19/2006 14:40'!
restartInvalidConnectionHandlers
	SqAudioChatServerFacade default restartInvalidConnectionHandlers! !

TdTimer subclass: #SqBoardThumbnailCache
	instanceVariableNames: 'cache lastEventNumbers'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqSquare-Server-Storage'!

!SqBoardThumbnailCache class methodsFor: 'constants' stamp: 'mu 5/26/2006 17:08'!
defaultIntervalSeconds
	^ 20! !

!SqBoardThumbnailCache class methodsFor: 'constants' stamp: 'mu 5/26/2006 15:57'!
defaultPriority
	^ Processor userBackgroundPriority + 1! !

!SqBoardThumbnailCache methodsFor: 'actions' stamp: 'mu 6/2/2006 10:25'!
at: boardId
	| value |
	value _ self cache at: boardId ifAbsent: [].
	value ifNil: [value _ self cache at: boardId put: (self latestPNGFileOf: boardId)].
	^value
	
	! !

!SqBoardThumbnailCache methodsFor: 'accessing' stamp: 'mu 5/26/2006 16:13'!
boardIds
	^SqServer default boardServers collect: [:each | each boardId]! !

!SqBoardThumbnailCache methodsFor: 'accessing' stamp: 'mu 5/26/2006 15:40'!
cache
	cache ifNil: [cache _ WeakValueDictionary new].
	^cache! !

!SqBoardThumbnailCache methodsFor: 'initialize-release' stamp: 'mu 6/6/2006 12:51'!
initialize
	super initialize.
	cache _ nil.
	lastEventNumbers _ nil! !

!SqBoardThumbnailCache methodsFor: 'accessing' stamp: 'mu 5/26/2006 16:17'!
lastEventNumbers
	lastEventNumbers ifNil: [lastEventNumbers _ Dictionary new].
	^lastEventNumbers! !

!SqBoardThumbnailCache methodsFor: 'private' stamp: 'mu 6/13/2006 21:35'!
latestPNGFileOf: boardId 
	| board |
	board _ SqSnapshotTaker boardAt: boardId.
	board
		ifNotNilDo: [:bo | ^ SqUtil thumbnailPNGFileOf: bo].
	^ [SqSnapshotReader latestPNGFileOf: boardId]
		on: Error
		do: [:ex | 
			SqWarning warnLog: 'Cannot get PNG:' , boardId asString.
			nil]! !

!SqBoardThumbnailCache methodsFor: 'actions' stamp: 'mu 5/26/2006 15:55'!
perform
	self updateThumbnails! !

!SqBoardThumbnailCache methodsFor: 'actions' stamp: 'mu 6/19/2006 15:11'!
removeKey: boardId
	self cache removeKey: boardId ifAbsent: [].
	
	! !

!SqBoardThumbnailCache methodsFor: 'actions' stamp: 'mu 6/13/2006 21:45'!
updateThumbnails
	self boardIds
		do: [:each | 
			| lastEventNumber currentEventNumber | 
			lastEventNumber _ self lastEventNumbers
						at: each
						ifAbsentPut: [0].
			currentEventNumber _ SqSnapshotReader latestEventNumberOf: each.
			currentEventNumber > lastEventNumber
				ifTrue: [self lastEventNumbers at: each put: currentEventNumber.
					self cache
						at: each
						put: (self latestPNGFileOf: each)]]! !

TdTimer subclass: #SqSnapshotDaemon
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqSquare-Server-Snapshot'!

!SqSnapshotDaemon class methodsFor: 'constants' stamp: 'mu 4/7/2006 14:53'!
defaultIntervalSeconds
	^ 30! !

!SqSnapshotDaemon class methodsFor: 'constants' stamp: 'mu 4/7/2006 16:05'!
defaultPriority
	^ Processor userSchedulingPriority -1 ! !

!SqSnapshotDaemon methodsFor: 'actions' stamp: 'mu 4/7/2006 16:05'!
perform
	"SqSnapshotDaemon default start" 
	"SqSnapshotDaemon default stop"

	self takeSnapshots! !

!SqSnapshotDaemon methodsFor: 'actions' stamp: 'mu 4/7/2006 14:37'!
takeSnapshots
	SqSettings isServer
		ifFalse: [^ self].
	SqServer default boardServers
		do: [:each | each takeSnapshot]! !

SBStyles subclass: #SqStyles
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqSquare-Server-WebUI'!

!SqStyles methodsFor: 'constants' stamp: 'mu 5/26/2006 12:25'!
body

^ '
body {
	margin: 0px; 
	font-family: sans-serif; 
	font-size: 11pt;
}

img{
	border-style: none ;
}

h1 {
	font-size: 180%;	
}

ul { 
	margin-left: 10px;
	padding-left: 10px;
}



'! !

!SqStyles methodsFor: 'constants' stamp: 'mu 4/12/2006 16:01'!
hierarchy

^ '
.shorePageHeader {
	color: black;
	margin: 0px; 
	padding: 0px;
	text-align: right;
	border: 0px;
	font: 16pt sans-serif;
	font-weight: bold;
	height: 50px;
	background-color: #E0FFFF;
	background-repeat: no-repeat;
	background-position: 4px;
}

.shorePageHeader img {
	float: left;
	border: 0px;
	padding-right: 50px;
}

.shorePageHeaderSubtitle {
	font: 9pt sans-serif;
	font-weight: bold;
}

'! !

!SqStyles methodsFor: 'constants' stamp: 'mu 4/12/2006 16:01'!
itemList

^ '
.list1 {
	color: #004180;
	list-style-type: circle;
	line-height: 2em; 
}
.list2 {
	color: cornflowerblue;
	list-style-type: none;
	line-height: 1em; 
}
'! !

!SqStyles methodsFor: 'constants' stamp: 'mu 5/26/2006 12:53'!
leftPanel
	
^'
.leftPanel {
	float: left;
	width: 160px;
	margin: 0px; 
	padding: 0px;
	height: 100%;
	background-color: white;
}


.leftPanelCollapsed {
	float: left;
	width: 15px;
	margin: 0px; 
	padding: 0px;
	height: 15px;
	background-color: white;
}

'! !

!SqStyles methodsFor: 'constants' stamp: 'mu 4/12/2006 16:01'!
list

^'
.ITEM-ROW { 
}

.ITEM-HEADER {
	color: gray;
}

.ITEM-DATA {
	color: black;
}

'! !

!SqStyles methodsFor: 'constants' stamp: 'mu 5/26/2006 19:54'!
main
	
^'
.main {
	width: 80%;
	padding: 0px;
	margin: 0px;
	margin-bottom: 0px;
	margin-left: 160px;
	padding-bottom: 20px;
}


.mainExpanded {
	width: 80%;
	padding: 0px;
	margin: 0px;
	margin-bottom: 0px;
	margin-left: 20px;
	height: 100%;
	padding-bottom: 20px;
}

'! !

!SqStyles methodsFor: 'constants' stamp: 'mu 5/26/2006 12:52'!
sideBarPanel

^ '

.sbSideBarSectionsTable{
	border-collapse: separate;
  	border-spacing: 10px 0px;
	table-layout: fixed;
	width: 160px;
}

.sbSideBarSectionsTableCollapsed{
	border-collapse: separate;
  	border-spacing: 10px 0px;
	table-layout: fixed;
	width: 15px;
}

.sbSideBarOpenButton {
	background-color: cornflowerblue;
	height: 15px;
	width: 15px;
	cursor: pointer;
	background-position: top right;
	background-repeat: no-repeat;
}

.sbSideBarCloseButton {
	background-color: cornflowerblue;
	height: 15px;
	width: 15px;
	cursor: pointer;
	background-position: top right;
	background-repeat: no-repeat;
}

.sbSideBarMenuHeader {
	background-color: #E0FFFF;
	color: rgb(33, 93, 198);
	font-weight: bold;
}

.sbSideBarMenuBody {
	background-color: #E0FFFF;
}


'! !

!SqStyles methodsFor: 'constants' stamp: 'mu 4/12/2006 16:01'!
table

^'
 .TABLE-EDITOR { 
	font-size: 12px;
	font-family: Arial;
	border: thin cornflowerblue solid; 
}
TH {
	font-weight: normal;
}
.TABLE-HEADER {
	font-size: 90%;
	border: none lightsteelblue solid;
	text-align: center;
}
.TABLE-DATA {
	border: thin lightsteelblue solid;
	text-align: center;
}
'! !
SqServer initialize!
SqServicePublisher initialize!
SqSnapshotTaker initialize!
SqAudioChatServerFacade initialize!
SqRpcDispatcher initialize!
